@@ -15,7 +15,7 @@
  * GNU General Public License for more details.
  *  
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  */
 
 /*
@@ -35,9 +35,8 @@ struct _CompletionData {
 
 CompletionData *completion_data_new(InternetAddress * ia,
                                     const gchar * nick_name);
-void completion_data_free(CompletionData * data);
+void completion_data_free(CompletionData * data, gpointer unused);
 gchar *completion_data_extract(CompletionData * data);
-gint address_compare(LibBalsaAddress *a, LibBalsaAddress *b);
 gint strncmp_word(const gchar * s1, const gchar * s2, gsize n);
 
 #endif
