@@ -35,6 +35,7 @@
 #include <glib/gi18n.h>
 #include "gmime-part-rfc2440.h"
 #include "libbalsa-gpgme.h"
+#include "html.h"
 
 LibBalsaMessageBody *
 libbalsa_message_body_new(LibBalsaMessage * message)
@@ -64,6 +65,22 @@ libbalsa_message_body_new(LibBalsaMessage * message)
     return body;
 }
 
+static void
+body_weak_notify(gpointer  data,
+                 GObject  *key)
+{
+    LibBalsaMessageBody *body = data;
+
+    g_hash_table_remove(body->selection_table, key);
+}
+
+static void
+selection_table_foreach(gpointer key,
+                        gpointer value,
+                        gpointer user_data)
+{
+    g_object_weak_unref(key, body_weak_notify, user_data);
+}
 
 void
 libbalsa_message_body_free(LibBalsaMessageBody * body)
@@ -92,8 +109,13 @@ libbalsa_message_body_free(LibBalsaMessageBody * body)
     libbalsa_message_body_free(body->parts);
 
     if (body->mime_part)
-	g_object_unref(body->mime_part);	
-    
+	g_object_unref(body->mime_part);
+
+    if (body->selection_table != NULL) {
+        g_hash_table_foreach(body->selection_table, selection_table_foreach, body);
+        g_hash_table_destroy(body->selection_table);
+    }
+
     g_free(body);
 }
 
@@ -194,8 +216,10 @@ libbalsa_message_body_set_message_part(LibBalsaMessageBody * body,
         body->embhdrs =
             libbalsa_message_body_extract_embedded_headers
             (embedded_message);
-        if (!*next_part)
+        if (!*next_part) {
             *next_part = libbalsa_message_body_new(body->message);
+            (*next_part)->parent = body;
+        }
         libbalsa_message_body_set_mime_body(*next_part,
                                             embedded_message->mime_part);
     }
@@ -214,8 +238,10 @@ libbalsa_message_body_set_multipart(LibBalsaMessageBody * body,
     count = g_mime_multipart_get_count (multipart);
     for (i = 0; i < count; i++) {
 	part = g_mime_multipart_get_part (multipart, i);
-	if (!*next_part)
+	if (!*next_part) {
 	    *next_part = libbalsa_message_body_new(body->message);
+	    (*next_part)->parent = body;
+	}
 	libbalsa_message_body_set_mime_body(*next_part, part);
 	next_part = &(*next_part)->next;
     }
@@ -303,7 +329,7 @@ libbalsa_message_body_type(LibBalsaMessageBody * body)
 }
 
 gchar *
-libbalsa_message_body_get_parameter(LibBalsaMessageBody * body,
+libbalsa_message_body_get_parameter(const LibBalsaMessageBody * body,
 				    const gchar * param)
 {
     GMimeContentType *type;
@@ -941,21 +967,254 @@ libbalsa_message_body_mp_related_root(LibBalsaMessageBody *body)
     return root_body;
 }
 
-LibBalsaMsgProtectState
-libbalsa_message_body_protect_state(const LibBalsaMessageBody *body)
+
+#ifdef HAVE_HTML_WIDGET
+
+/** @brief Find the multipart/alternative parent of a body
+ *
+ * @param[in] body message body
+ * @return the body's multipart/alternative parent, NULL if it is not the child of such a part
+ *
+ * Return the multipart/alternative parent of the passed body, which @em must be either the direct parent, or if the body's direct
+ * parent is a multipart/related the direct parent of the latter.
+ */
+static inline LibBalsaMessageBody *
+find_mp_alt_parent(const LibBalsaMessageBody *body)
 {
-	LibBalsaMsgProtectState state;
+	LibBalsaMessageBody *mp_alt = NULL;
+
+	mp_alt = body->parent;
+	if ((mp_alt != NULL) && (mp_alt->body_type == LIBBALSA_MESSAGE_BODY_TYPE_MULTIPART)) {
+		gchar *conttype;
+
+		conttype = libbalsa_message_body_get_mime_type(mp_alt);
+		if (strcmp(conttype, "multipart/related") == 0) {
+			g_free(conttype);
+			mp_alt = mp_alt->parent;
+			if (mp_alt != NULL) {
+				conttype = libbalsa_message_body_get_mime_type(mp_alt);
+			} else {
+				conttype = NULL;
+			}
+		}
+
+		if ((conttype != NULL) && strcmp(conttype, "multipart/alternative") != 0) {
+			mp_alt = NULL;
+		}
+		g_free(conttype);
+	}
+
+	return mp_alt;
+}
+
+/** @brief Set if a multipart/alternative HTML or plain part is selected
+ *
+ * @param[in] body message body
+ *
+ * Iff the passed body is the child of a multipart/alternative set the LibBalsaMessageBody::html_selected property of the latter if
+ * the body's content type is a HTML type.
+ *
+ * @sa find_mp_alt_parent(), libbalsa_html_type()
+ */
+void
+libbalsa_message_body_set_mp_alt_selection(LibBalsaMessageBody *body, gpointer key)
+{
+	LibBalsaMessageBody *mp_alt_body;
+
+	if ((body != NULL) && (body->body_type == LIBBALSA_MESSAGE_BODY_TYPE_TEXT)) {
+		mp_alt_body = find_mp_alt_parent(body);
+		if (mp_alt_body != NULL) {
+			gchar *conttype;
+                        LibBalsaMpAltSelection selection;
+
+			conttype = libbalsa_message_body_get_mime_type(body);
+			if (libbalsa_html_type(conttype) != LIBBALSA_HTML_TYPE_NONE) {
+				selection = LIBBALSA_MP_ALT_HTML;
+			} else {
+				selection = LIBBALSA_MP_ALT_PLAIN;
+			}
+			g_free(conttype);
+
+                        /* Remember the most recent selection: */
+                        mp_alt_body->mp_alt_selection = selection;
+
+                        if (mp_alt_body->selection_table == NULL)
+                            mp_alt_body->selection_table = g_hash_table_new(NULL, NULL);
+
+                        /* Remember the most recent selection for this key: */
+                        if (g_hash_table_insert(mp_alt_body->selection_table, key,
+                                                GINT_TO_POINTER(selection))) {
+                            g_object_weak_ref(key, body_weak_notify, mp_alt_body);
+                        }
+		}
+	}
+}
+
+static inline gboolean body_is_type(const LibBalsaMessageBody *body,
+                                    const gchar               *type,
+                                    const gchar               *sub_type);
+
+/** @brief Check if a multipart/alternative HTML or plain part is selected
+ *
+ * @param[in] body message body
+ * @return which part of a multipart/alternative is selected, @ref LIBBALSA_MP_ALT_AUTO if @em body id not part of a
+ *         multipart/alternative or if the selection shall be done automatically
+ * @sa find_mp_alt_parent()
+ */
+LibBalsaMpAltSelection
+libbalsa_message_body_get_mp_alt_selection(LibBalsaMessageBody *body, gpointer key)
+{
+	LibBalsaMpAltSelection selection;
+
+	g_return_val_if_fail(body != NULL, LIBBALSA_MP_ALT_AUTO);
+
+	if (!body_is_type(body, "multipart", "alternative"))
+            body = find_mp_alt_parent(body);
+
+	if (body == NULL) {
+            selection = LIBBALSA_MP_ALT_AUTO;
+        } else {
+            if (body->selection_table != NULL && g_hash_table_contains(body->selection_table, key)) {
+                /* The part is currently being viewed, so return the
+                 * selection that was used to view it: */
+		selection = GPOINTER_TO_INT(g_hash_table_lookup(body->selection_table, key));
+            } else {
+                /* The part is not currently being viewed, so return the
+                 * most recent selection: */
+                selection = body->mp_alt_selection;
+            }
+        }
+
+	return selection;
+}
+
+#endif /*HAVE_HTML_WIDGET*/
+
+
+/** Basic requirements for a multipart crypto: protocol is not NULL, exactly two body parts */
+#define MP_CRYPT_STRUCTURE(part, protocol)										\
+	(((protocol) != NULL) && ((body)->parts != NULL) && 						\
+	 ((body)->parts->next != NULL) && ((body)->parts->next->next == NULL))
+
+/** Check if protocol is application/subtype, and that part has the same MIME type */
+#define IS_PROT_PART(part, protocol, subtype)									\
+	((g_ascii_strcasecmp("application/" subtype, protocol) == 0) &&				\
+	 body_is_type(part, "application", subtype))
+
+static inline gboolean
+body_is_type(const LibBalsaMessageBody *body,
+			 const gchar               *type,
+			 const gchar               *sub_type)
+{
+	gboolean retval;
+	GMimeContentType *content_type;
+
+	if (body->mime_part) {
+		content_type = g_mime_object_get_content_type(body->mime_part);
+		retval = g_mime_content_type_is_type(content_type, type, sub_type);
+	} else {
+		content_type = g_mime_content_type_parse(libbalsa_parser_options(), body->content_type);
+		retval = g_mime_content_type_is_type(content_type, type, sub_type);
+		g_object_unref(content_type);
+	}
+
+	return retval;
+}
+
+/** @brief Get the basic protection mode of a body
+ *
+ * @param body message body
+ * @return a bit mask indicating the protection state of the body
+ *
+ * Check the structure and parameters of the passed body, and return a bit mask indicating if it is signed or encrypted, and which
+ * protocol is used.  @ref LIBBALSA_PROTECT_ERROR indicates obvious errors.  The bit mask does @em not include the validity of a
+ * signature (see libbalsa_message_body_signature_state() for this purpose).
+ */
+guint
+libbalsa_message_body_protect_mode(const LibBalsaMessageBody * body)
+{
+	guint result;
+
+	g_return_val_if_fail(body != NULL, 0);
+	g_return_val_if_fail(body->content_type != NULL, 0);
+
+	if (body_is_type(body, "multipart", "signed")) {
+		/* multipart/signed (PGP/MIME, S/MIME) must have a protocol and a micalg parameter */
+		gchar *protocol = libbalsa_message_body_get_parameter(body, "protocol");
+		gchar *micalg = libbalsa_message_body_get_parameter(body, "micalg");
+
+		result = LIBBALSA_PROTECT_SIGN;
+		if (MP_CRYPT_STRUCTURE(body, protocol)) {
+			if (IS_PROT_PART(body->parts->next, protocol, "pkcs7-signature") ||
+				IS_PROT_PART(body->parts->next, protocol, "x-pkcs7-signature")) {
+				result |= LIBBALSA_PROTECT_SMIME;
+				if (micalg == NULL) {
+					result |= LIBBALSA_PROTECT_ERROR;
+				}
+			} else if (IS_PROT_PART(body->parts->next, protocol, "pgp-signature")) {
+				result |= LIBBALSA_PROTECT_RFC3156;
+				if ((micalg == NULL) || (g_ascii_strncasecmp("pgp-", micalg, 4) != 0)) {
+					result |= LIBBALSA_PROTECT_ERROR;
+				}
+			} else {
+				result |= LIBBALSA_PROTECT_ERROR;
+			}
+		} else {
+			result |= LIBBALSA_PROTECT_ERROR;
+		}
+		g_free(micalg);
+		g_free(protocol);
+	} else if (body_is_type(body, "multipart", "encrypted")) {
+		/* multipart/signed (PGP/MIME, S/MIME) must have a protocol parameter */
+		gchar *protocol = libbalsa_message_body_get_parameter(body, "protocol");
+
+		result = LIBBALSA_PROTECT_ENCRYPT | LIBBALSA_PROTECT_RFC3156;
+		if (!MP_CRYPT_STRUCTURE(body, protocol) ||
+			!IS_PROT_PART(body->parts, protocol, "pgp-encrypted") ||
+			!body_is_type(body->parts->next, "application", "octet-stream")) {
+			result |= LIBBALSA_PROTECT_ERROR;
+		}
+		g_free(protocol);
+	} else if (body_is_type(body, "application", "pkcs7-mime") ||
+			   body_is_type(body, "application", "x-pkcs7-mime")) {
+		/* multipart/pkcs7-mime (S/MIME) must have a smime-type parameter */
+		gchar *smime_type = libbalsa_message_body_get_parameter(body, "smime-type");
+
+		result = LIBBALSA_PROTECT_SMIME;
+		if ((g_ascii_strcasecmp("enveloped-data", smime_type) == 0) ||
+			(g_ascii_strcasecmp("signed-data", smime_type) == 0)) {
+			result |= LIBBALSA_PROTECT_ENCRYPT;
+		} else {
+			result |= LIBBALSA_PROTECT_ERROR;
+		}
+		g_free(smime_type);
+	} else {
+		result = LIBBALSA_PROTECT_NONE;
+	}
+
+	return result;
+}
+
+/** @brief Get the cryptographic signature state of a body
+ *
+ * @param body message body
+ * @return a value indicating the cryptographic signature state of the body
+ */
+guint
+libbalsa_message_body_signature_state(const LibBalsaMessageBody *body)
+{
+	guint state;
 
 	if ((body == NULL) || (body->sig_info == NULL) ||
 		(g_mime_gpgme_sigstat_status(body->sig_info) == GPG_ERR_NOT_SIGNED) ||
 		(g_mime_gpgme_sigstat_status(body->sig_info) == GPG_ERR_CANCELED)) {
-		state = LIBBALSA_MSG_PROTECT_NONE;
+		state = LIBBALSA_PROTECT_NONE;
 	} else if (g_mime_gpgme_sigstat_status(body->sig_info) != GPG_ERR_NO_ERROR) {
-		state = LIBBALSA_MSG_PROTECT_SIGN_BAD;
+		state = LIBBALSA_PROTECT_SIGN_BAD;
 	} else if ((g_mime_gpgme_sigstat_summary(body->sig_info) & GPGME_SIGSUM_VALID) == GPGME_SIGSUM_VALID) {
-		state = LIBBALSA_MSG_PROTECT_SIGN_GOOD;
+		state = LIBBALSA_PROTECT_SIGN_GOOD;
 	} else {
-		state = LIBBALSA_MSG_PROTECT_SIGN_NOTRUST;
+		state = LIBBALSA_PROTECT_SIGN_NOTRUST;
 	}
 
 	return state;
@@ -977,10 +1236,10 @@ libbalsa_message_body_multipart_signed(const LibBalsaMessageBody *body)
 			(body->content_type != NULL) &&
 			(g_ascii_strcasecmp(body->content_type, "multipart/signed") == 0) &&
 			(body->parts != NULL) &&					/* must have children */
-		    (body->parts->next != NULL) &&				/* must have *two* child parts... */
-		    (body->parts->next->next == NULL) &&		/* ...but not more */
-		    (body->parts->next->sig_info != NULL) &&
-		    (g_mime_gpgme_sigstat_status(body->parts->next->sig_info) != GPG_ERR_NOT_SIGNED);
+			(body->parts->next != NULL) &&				/* must have *two* child parts... */
+			(body->parts->next->next == NULL) &&		/* ...but not more */
+			(body->parts->next->sig_info != NULL) &&
+			(g_mime_gpgme_sigstat_status(body->parts->next->sig_info) != GPG_ERR_NOT_SIGNED);
 }
 
 
