@@ -33,7 +33,9 @@
 
 #include <stdio.h>
 #include <string.h>
+#include <unistd.h>
 #include <glib/gi18n.h>
+#include <glib/gstdio.h>
 
 #ifdef HAVE_HTML_WIDGET
 
@@ -43,6 +45,15 @@
 #define G_LOG_DOMAIN "html"
 
 
+/* Note:
+ * If the following variable is set, search the build folder for the Webkit
+ * extension first, and fall back to the install folder if the folder does
+ * not exist.  Thus, this variable should be set for testing/development,
+ * and unset for release builds to avoid leaking the build folder name.
+ */
+#define WEB_EXT_CHECK_BUILDDIR	1
+
+
 #define CID_REGEX	"<[^>]*src\\s*=\\s*['\"]?\\s*cid:"
 /* use a negative lookahead assertion to match "src=" *not* followed by "cid:" */
 #define SRC_REGEX	"<[^>]*src\\s*=\\s*(?!['\"]?\\s*cid:)"
@@ -59,6 +70,10 @@
 #define LBH_NATURAL_SIZE (-1)
 
 
+/* indicates if Balsa's HTML filter webkit extension is available */
+static guint html_filter_found = 0U;
+
+
 /*
  * lbh_get_body_content
  *
@@ -77,7 +92,7 @@ lbh_get_body_content(LibBalsaMessageBody * body, gchar ** buf)
     if (len < 0) {
         libbalsa_information(LIBBALSA_INFORMATION_ERROR,
                              _("Could not get an HTML part: %s"),
-                             err ? err->message : "Unknown error");
+                             err ? err->message : _("Unknown error"));
         g_error_free(err);
         return len;
     }
@@ -135,7 +150,7 @@ html2text(gchar ** text, gsize len)
     if (err != NULL) {
         libbalsa_information(LIBBALSA_INFORMATION_ERROR,
                              _("Could not convert HTML part to text: %s"),
-                             err ? err->message : "Unknown error");
+                             err->message);
         g_error_free(err);
     }
 }
@@ -175,6 +190,17 @@ typedef struct {
 
 #define LIBBALSA_HTML_INFO "libbalsa-webkit2-info"
 
+
+static WebKitWebContext *lbh_get_web_view_context(void);
+
+
+void
+libbalsa_html_init(void)
+{
+	(void) lbh_get_web_view_context();
+}
+
+
 /*
  * Unlike older HTML widgets, webkit2 wants UTF-8 text
  */
@@ -361,33 +387,52 @@ lbh_decide_policy_cb(WebKitWebView           * web_view,
     return TRUE;
 }
 
+
+static void
+lbh_load_external_resources(WebKitWebView *web_view, gboolean load_resources)
+{
+	WebKitUserMessage *message;
+	GVariant *data;
+
+	data = g_variant_new_boolean(load_resources);
+	message = webkit_user_message_new("load_ext", data);
+	webkit_web_view_send_message_to_page(web_view, message, NULL, NULL, NULL);
+	g_usleep(1000);
+}
+
+
 /*
- * Show the GtkInfoBar for asking about downloading images
+ * Show the GtkInfoBar for asking about downloading external content
  *
  * First two signal callbacks
  */
 
 static void
 lbh_info_bar_response_cb(GtkInfoBar * info_bar,
-                         gint response_id, gpointer data)
+	gint response_id, gpointer data)
 {
-    LibBalsaWebKitInfo *info = data;
+	LibBalsaWebKitInfo *info = data;
 
-    if (response_id == GTK_RESPONSE_OK) {
-        gchar *text;
+	if (response_id == GTK_RESPONSE_OK) {
+		gchar *text;
 
-        if (lbh_get_body_content_utf8(info->body, &text) >= 0) {
-            WebKitSettings *settings;
+		if (lbh_get_body_content_utf8(info->body, &text) >= 0) {
+			if (g_atomic_int_get(&html_filter_found) != 0) {
+				lbh_load_external_resources(info->web_view, TRUE);
+				info->body->html_ext_loaded= TRUE;
+			} else {
+				WebKitSettings *settings;
 
-            settings = webkit_web_view_get_settings(info->web_view);
-            webkit_settings_set_auto_load_images(settings, TRUE);
-            webkit_web_view_load_html(info->web_view, text, NULL);
-            g_free(text);
-        }
-    }
+				settings = webkit_web_view_get_settings(info->web_view);
+				webkit_settings_set_auto_load_images(settings, TRUE);
+			}
+			webkit_web_view_load_html(info->web_view, text, NULL);
+			g_free(text);
+		}
+	}
 
-    gtk_widget_destroy(info->info_bar);
-    info->info_bar = NULL;
+	gtk_widget_destroy(info->info_bar);
+	info->info_bar = NULL;
 }
 
 static void
@@ -403,26 +448,13 @@ lbh_info_bar(LibBalsaWebKitInfo * info)
     GtkInfoBar *info_bar;
     GtkWidget *label;
     GtkWidget *content_area;
-#ifdef GTK_INFO_BAR_WRAPPING_IS_BROKEN
     static const gchar text[] =
-                 N_("This message part contains images "
-                    "from a remote server.\n"
-                    "To protect your privacy, "
-                    "Balsa has not downloaded them.\n"
-                    "You may choose to download them "
-                    "if you trust the server.");
-#else                           /* GTK_INFO_BAR_WRAPPING_IS_BROKEN */
-    static const gchar text[] =
-                 N_("This message part contains images "
-                    "from a remote server. "
-                    "To protect your privacy, "
-                    "Balsa has not downloaded them. "
-                    "You may choose to download them "
-                    "if you trust the server.");
-#endif                          /* GTK_INFO_BAR_WRAPPING_IS_BROKEN */
+                 N_("This message part references content on one or more external servers. "
+                	"To protect your privacy, Balsa has not downloaded it. You may choose "
+                	"to download it if you trust the sender of the message.");
 
     info_bar_widget =
-        gtk_info_bar_new_with_buttons(_("_Download images"),
+        gtk_info_bar_new_with_buttons(_("_Download external content"),
                                      GTK_RESPONSE_OK,
                                      _("_Close"), GTK_RESPONSE_CLOSE,
                                      NULL);
@@ -454,14 +486,15 @@ lbh_resource_notify_response_cb(WebKitWebResource * resource,
                                 GParamSpec        * pspec,
                                 gpointer            data)
 {
-    LibBalsaWebKitInfo *info = data;
+    LibBalsaWebKitInfo *info = (LibBalsaWebKitInfo *) data;
     const gchar *mime_type;
     WebKitURIResponse *response;
 
     response = webkit_web_resource_get_response(resource);
     mime_type = webkit_uri_response_get_mime_type(response);
     g_debug("%s mime-type %s", __func__, mime_type);
-    if (g_ascii_strncasecmp(mime_type, "image/", 6) != 0)
+    if ((g_atomic_int_get(&html_filter_found) != 0) ||
+    	(g_ascii_strncasecmp(mime_type, "image/", 6) != 0))
         return;
 
     if (info->info_bar) {
@@ -577,18 +610,45 @@ lbh_context_menu_cb(WebKitWebView       * web_view,
     GtkWidget *parent;
     gboolean retval;
 
+    /* display the default Webkit context menu if the user selected some content */
+    if (webkit_hit_test_result_context_is_selection(hit_test_result)) {
+        return FALSE;
+    }
+
     parent = gtk_widget_get_parent(GTK_WIDGET(web_view));
     /* The signal is asynchronous, so gtk_get_current_event() gets NULL;
      * we pass the event to the popup-menu handler: */
     g_object_set_data(G_OBJECT(parent), LIBBALSA_HTML_POPUP_EVENT, event);
+    g_object_set_data(G_OBJECT(parent), LIBBALSA_HTML_POPUP_URL,
+        (gpointer) webkit_hit_test_result_get_link_uri(hit_test_result));
 
     g_signal_emit_by_name(parent, "popup-menu", &retval);
 
     g_object_set_data(G_OBJECT(parent), LIBBALSA_HTML_POPUP_EVENT, NULL);
+    g_object_set_data(G_OBJECT(parent), LIBBALSA_HTML_POPUP_URL, NULL);
 
     return retval;
 }
 
+static gboolean
+lbh_web_extension_cb(WebKitWebContext G_GNUC_UNUSED *context,
+					 WebKitUserMessage              *message,
+					 gpointer G_GNUC_UNUSED          user_data)
+{
+	gboolean result;
+
+	if (strcmp(webkit_user_message_get_name(message), "balsa-html-filter") == 0) {
+		g_atomic_int_or(&html_filter_found, 1U);
+		g_debug("%s: Balsa HTML filter WebKit extension found", __func__);
+		result = TRUE;
+	} else {
+		g_info("%s: unknown webkit extension message '%s'", __func__, webkit_user_message_get_name(message));
+		result = FALSE;
+	}
+	return result;
+}
+
+
 static WebKitWebContext *
 lbh_get_web_view_context(void)
 {
@@ -596,16 +656,58 @@ lbh_get_web_view_context(void)
 
 	if (g_once_init_enter(&context)) {
 		WebKitWebsiteDataManager *data_manager;
+		WebKitCookieManager *cookie_manager;
 		WebKitWebContext *tmp;
+		WebKitWebView *view;
 		gchar *cache_dir;
+		WebKitSettings *settings;
 
 		cache_dir = g_build_filename(g_get_home_dir(), ".balsa", CACHE_DIR, NULL);
 		data_manager = webkit_website_data_manager_new("base-cache-directory", cache_dir, NULL);
 		g_free(cache_dir);
+		webkit_website_data_manager_set_tls_errors_policy(data_manager, WEBKIT_TLS_ERRORS_POLICY_FAIL);
+		cookie_manager = webkit_website_data_manager_get_cookie_manager(data_manager);
+		webkit_cookie_manager_set_accept_policy(cookie_manager, WEBKIT_COOKIE_POLICY_ACCEPT_NEVER);
 		tmp = webkit_web_context_new_with_website_data_manager(data_manager);
-		webkit_web_context_set_cache_model(tmp, WEBKIT_CACHE_MODEL_DOCUMENT_BROWSER);
+#ifdef WEB_EXT_CHECK_BUILDDIR
+		g_debug("%s: WEB_EXT_CHECK_BUILDDIR is defined, check for '%s'", __func__, BALSA_WEB_EXT_DEVEL "/libhtmlfilter.so");
+		if (g_access(BALSA_WEB_EXT_DEVEL "/libhtmlfilter.so", R_OK) == 0) {
+			g_debug("%s: set extensions folder '%s'", __func__, BALSA_WEB_EXT_DEVEL);
+			webkit_web_context_set_web_extensions_directory(tmp, BALSA_WEB_EXT_DEVEL);
+		} else {
+			g_debug("%s: set extensions folder '%s'", __func__, BALSA_WEB_EXTENSIONS);
+			webkit_web_context_set_web_extensions_directory(tmp, BALSA_WEB_EXTENSIONS);
+		}
+#else
+		g_debug("%s: set extensions folder '%s'", __func__, BALSA_WEB_EXTENSIONS);
+		webkit_web_context_set_web_extensions_directory(tmp, BALSA_WEB_EXTENSIONS);
+#endif
+		g_signal_connect(tmp, "user-message-received", G_CALLBACK(lbh_web_extension_cb), NULL);
+		webkit_web_context_set_cache_model(tmp, WEBKIT_CACHE_MODEL_WEB_BROWSER);
 		webkit_web_context_register_uri_scheme(tmp, "cid", lbh_cid_cb, NULL, NULL);
 		g_debug("%s: registered “cid:” scheme", __func__);
+
+		/* create a dummy view to trigger loading the html filter extension */
+		view = WEBKIT_WEB_VIEW(webkit_web_view_new_with_context(tmp));
+		settings = webkit_web_view_get_settings(view);
+		/* note: enabling hardware acceleration seems to trigger strange segfaults in Webkit at least on some systems running
+		 * Wayland (probably the same issue as https://github.com/NixOS/nixpkgs/issues/168939), so we just disable it
+		 * completely */
+		webkit_settings_set_hardware_acceleration_policy(settings, WEBKIT_HARDWARE_ACCELERATION_POLICY_NEVER);
+		webkit_web_view_load_uri(view, "about:blank");
+		while (webkit_web_view_is_loading(view)) {
+			gtk_main_iteration_do(FALSE);
+			g_usleep(100);
+		}
+		g_object_ref_sink(view);
+		g_object_unref(view);
+		if (g_atomic_int_get(&html_filter_found) != 0) {
+			g_debug("%s: Balsa HTML filter available", __func__);
+		} else {
+			libbalsa_information(LIBBALSA_INFORMATION_WARNING,
+				_("Balsa's external HTML resources filter web extension is not available in the folder “%s”, "
+				  "falling back to simplified image filtering. Please check your installation. "), BALSA_WEB_EXTENSIONS);
+		}
 		g_once_init_leave(&context, tmp);
 	}
 	return context;
@@ -614,7 +716,7 @@ lbh_get_web_view_context(void)
 static WebKitWebView *
 lbh_web_view_new(LibBalsaWebKitInfo *info,
 				 gint				 width,
-				 gboolean            auto_load_images)
+				 gboolean            auto_load_ext_content)
 {
 	WebKitWebView *view;
 	WebKitSettings *settings;
@@ -625,6 +727,8 @@ lbh_web_view_new(LibBalsaWebKitInfo *info,
     gtk_widget_set_vexpand(GTK_WIDGET(view), TRUE);
 
 	settings = webkit_web_view_get_settings(view);
+	/* might be paranoid - see note in function lbh_get_web_view_context() above */
+	webkit_settings_set_hardware_acceleration_policy(settings, WEBKIT_HARDWARE_ACCELERATION_POLICY_NEVER);
 #if WEBKIT_CHECK_VERSION(2, 31, 91)
     g_object_set(G_OBJECT(settings), "enable-plugins", FALSE, NULL);
 #else  /* WEBKIT_CHECK_VERSION(2, 31, 91) */
@@ -633,7 +737,9 @@ lbh_web_view_new(LibBalsaWebKitInfo *info,
     webkit_settings_set_enable_javascript(settings, FALSE);
 	webkit_settings_set_enable_java(settings, FALSE);
 	webkit_settings_set_enable_hyperlink_auditing(settings, TRUE);
-	webkit_settings_set_auto_load_images(settings, auto_load_images);
+	webkit_settings_set_auto_load_images(settings,
+		auto_load_ext_content || (g_atomic_int_get(&html_filter_found) != 0));
+	lbh_load_external_resources(view, auto_load_ext_content);
 
 	g_signal_connect(view, "web-process-terminated", G_CALLBACK(lbh_web_process_terminated_cb), info);
     g_signal_connect(view, "decide-policy", G_CALLBACK(lbh_decide_policy_cb), info);
@@ -669,13 +775,12 @@ dump_snapshot(GObject      *source_object,
  *
  * \param body HTML message body part
  * \param width rendering width in Cairo units (1/72")
- * \param load_external_images whether external images referenced by the HTML shall be loaded
+ * \param load_external_content whether external content referenced by the HTML shall be loaded
  * \return a cairo surface on success, or NULL on error
  */
 cairo_surface_t *
 libbalsa_html_print_bitmap(LibBalsaMessageBody *body,
-						   gdouble 				width,
-						   gboolean 			load_external_images)
+						   gdouble 				width)
 {
 	gint render_width;
     gchar *text;
@@ -702,7 +807,7 @@ libbalsa_html_print_bitmap(LibBalsaMessageBody *body,
 	render_width = (gint) (width * HTML_PRINT_DPI / 72.0);
 	g_debug("%s: request Cairo width %g, render width %d", __func__, width, render_width);
     gtk_window_set_default_size(GTK_WINDOW(offline_window), render_width, LBH_NATURAL_SIZE);
-    view = lbh_web_view_new(info, render_width, load_external_images || (have_src_cid && !have_src_oth));
+    view = lbh_web_view_new(info, render_width, body->html_ext_loaded || (have_src_cid && !have_src_oth));
     webkit_web_view_set_zoom_level(view, HTML_PRINT_ZOOM);			/* heuristic setting, any way to calculate it? */
     gtk_container_add(GTK_CONTAINER(offline_window), GTK_WIDGET(view));
     gtk_widget_show_all(offline_window);
@@ -751,7 +856,7 @@ GtkWidget *
 libbalsa_html_new(LibBalsaMessageBody * body,
                   LibBalsaHtmlCallback  hover_cb,
                   LibBalsaHtmlCallback  clicked_cb,
-                  gboolean              auto_load_images)
+                  gboolean              auto_load_ext_content)
 {
     gchar *text;
     gssize len;
@@ -772,8 +877,9 @@ libbalsa_html_new(LibBalsaMessageBody * body,
     have_src_cid = g_regex_match_simple(CID_REGEX, text, G_REGEX_CASELESS, 0);
     have_src_oth = g_regex_match_simple(SRC_REGEX, text, G_REGEX_CASELESS, 0);
 
+    body->html_ext_loaded = auto_load_ext_content || body->html_ext_loaded;
     info->web_view = lbh_web_view_new(info, LBH_NATURAL_SIZE,
-    	auto_load_images || (have_src_cid && !have_src_oth));
+    	body->html_ext_loaded || (have_src_cid && !have_src_oth));
 
     g_signal_connect(info->web_view, "mouse-target-changed",
                      G_CALLBACK(lbh_mouse_target_changed_cb), info);
@@ -784,7 +890,7 @@ libbalsa_html_new(LibBalsaMessageBody * body,
     gtk_box_pack_end(GTK_BOX(vbox), GTK_WIDGET(info->web_view), TRUE, TRUE, 0);
 
     /* Simple check for possible resource requests: */
-    if (have_src_oth) {
+    if (have_src_oth && !body->html_ext_loaded) {
         info->info_bar = lbh_info_bar(info);
         gtk_box_pack_start(GTK_BOX(vbox), info->info_bar, FALSE, FALSE, 0);
         g_debug("%s shows info_bar", __func__);
