@@ -14,7 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  */
 
 
@@ -78,9 +78,13 @@ static gchar *utf8_valid_str(const char *gpgme_str)
 static const gchar *get_utf8_locale(int category);
 #endif
 
+static void gpg_check_capas(const gchar *gpg_path,
+							const gchar *version);
+
 
 static gboolean has_proto_openpgp = FALSE;
 static gboolean has_proto_cms = FALSE;
+static gpg_capabilities gpg_capas;
 
 static gpgme_passphrase_cb_t gpgme_passphrase_cb = NULL;
 static lbgpgme_select_key_cb select_key_cb = NULL;
@@ -126,6 +130,9 @@ libbalsa_gpgme_init(gpgme_passphrase_cb_t get_passphrase,
 		g_debug("protocol %s: engine %s (home %s, version %s)",
 		      gpgme_get_protocol_name(e->protocol),
 		      e->file_name, e->home_dir, e->version);
+		if (e->protocol == GPGME_PROTOCOL_OpenPGP) {
+			gpg_check_capas(e->file_name, e->version);
+		}
 	    e = e->next;
 	}
     }
@@ -163,6 +170,28 @@ libbalsa_gpgme_init(gpgme_passphrase_cb_t get_passphrase,
 }
 
 
+/** \brief Return the protocol as human-readable string
+ *
+ * \param protocol Protocol to return as string.
+ * \return a static string denoting the passed protocol, or "unknown".
+ *
+ * Note that this function differs from gegme_get_protocol_name, as it returns "S/MIME" instead of "CMS" for GPGME_PROTOCOL_CMS,
+ * and "unknown" for all other protocols but GPGME_PROTOCOL_OpenPGP (which is returned as "OpenPGP").
+ */
+const gchar *
+libbalsa_gpgme_protocol_name(gpgme_protocol_t protocol)
+{
+    switch (protocol) {
+    case GPGME_PROTOCOL_OpenPGP:
+    	return _("OpenPGP");
+    case GPGME_PROTOCOL_CMS:
+    	return _("S/MIME");
+    default:
+    	return _("unknown");
+    }
+}
+
+
 /** \brief Check if a crypto engine is available
  *
  * \param protocol Protocol for which the engine is checked.
@@ -184,6 +213,24 @@ libbalsa_gpgme_check_crypto_engine(gpgme_protocol_t protocol)
 }
 
 
+/** \brief Get capabilities of the gpg engine
+ *
+ * \return a pointer to the capabilities of the GnuPG engine, or NULL if it is not supported
+ *
+ * If an engine for the OpenPGP protocol is available, return a structure containing the path of the executable, and information if
+ * some \em export-filter options are available.  This information is needed to export a minimal Autocrypt key, but unfortunately
+ * cannot be determined from the engine version.
+ *
+ * \sa libbalsa_gpgme_export_autocrypt_key(), gpg_check_capas()
+ * \todo Actually, gpgme should provide a minimalistic key export.
+ */
+const gpg_capabilities *
+libbalsa_gpgme_gpg_capabilities(void)
+{
+	return has_proto_openpgp ? &gpg_capas : NULL;
+}
+
+
 /** \brief Create a new GpgME context for a protocol
  *
  * \param protocol requested protocol
@@ -210,7 +257,7 @@ libbalsa_gpgme_new_with_proto(gpgme_protocol_t        protocol,
 	} else {
 		err = gpgme_set_protocol(ctx, protocol);
 		if (err != GPG_ERR_NO_ERROR) {
-			libbalsa_gpgme_set_error(error, err, _("could not set protocol “%s”"), gpgme_get_protocol_name(protocol));
+			libbalsa_gpgme_set_error(error, err, _("could not set protocol “%s”"), libbalsa_gpgme_protocol_name(protocol));
 		    gpgme_release(ctx);
 		    ctx = NULL;
 		} else {
@@ -265,7 +312,7 @@ libbalsa_gpgme_ctx_set_home(gpgme_ctx_t   ctx,
     		result = TRUE;
     	} else {
     		libbalsa_gpgme_set_error(error, err, _("could not set folder “%s” for engine “%s”"), home_dir,
-    			gpgme_get_protocol_name(protocol));
+    				libbalsa_gpgme_protocol_name(protocol));
     	}
     }
 
@@ -347,7 +394,7 @@ libbalsa_gpgme_verify(GMimeStream * content, GMimeStream * sig_plain,
     	libbalsa_gpgme_set_error(error, err,
 			       _("signature verification failed"));
 	result = g_mime_gpgme_sigstat_new(ctx);
-	result->status = err;
+	g_mime_gpgme_sigstat_set_status(result, err);
     } else
 	result = g_mime_gpgme_sigstat_new_from_gpgme_ctx(ctx);
 
@@ -746,8 +793,10 @@ libbalsa_gpgme_get_seckey(gpgme_protocol_t   protocol,
 	if (ctx != NULL) {
 		GList *keys = NULL;
 
-		/* let gpgme list all available keys */
-		if (libbalsa_gpgme_list_keys(ctx, &keys, NULL, name, TRUE, FALSE, error)) {
+		/* Let gpgme list all available secret keys, including those not matching the passed email address.
+		 * Rationale: enable selecting a secret key even if the local email address is re-written by the MTA.
+		 * See e.g. http://www.postfix.org/ADDRESS_REWRITING_README.html#generic */
+		if (libbalsa_gpgme_list_keys(ctx, &keys, NULL, NULL, TRUE, FALSE, FALSE, error)) {
 			if (keys != NULL) {
 				gpgme_key_t key;
 
@@ -773,7 +822,7 @@ libbalsa_gpgme_get_seckey(gpgme_protocol_t   protocol,
 					GTK_DIALOG_DESTROY_WITH_PARENT | libbalsa_dialog_flags(),
 					GTK_MESSAGE_INFO, GTK_BUTTONS_CLOSE,
 					_("No private key for protocol %s is available for the signer “%s”"),
-					gpgme_get_protocol_name(protocol), name);
+					libbalsa_gpgme_protocol_name(protocol), name);
 				(void) gtk_dialog_run(GTK_DIALOG(dialog));
 				gtk_widget_destroy(dialog);
 			}
@@ -913,7 +962,7 @@ get_key_from_name(gpgme_ctx_t   ctx,
 	}
 
 	/* let gpgme list keys */
-	list_res = libbalsa_gpgme_list_keys(ctx, &keys, &bad_keys, mail_name, secret, FALSE, error);
+	list_res = libbalsa_gpgme_list_keys(ctx, &keys, &bad_keys, mail_name, secret, FALSE, FALSE, error);
 	g_free(mail_name);
 	if (!list_res) {
 		return GPG_ERR_GENERAL;
@@ -992,7 +1041,7 @@ get_pubkey(gpgme_ctx_t   ctx,
 	gpgme_key_t key = NULL;
 
 	/* let gpgme list all available keys */
-	if (libbalsa_gpgme_list_keys(ctx, &keys, NULL, NULL, FALSE, FALSE, error)) {
+	if (libbalsa_gpgme_list_keys(ctx, &keys, NULL, NULL, FALSE, FALSE, FALSE, error)) {
 		if (keys != NULL) {
 			/* let the user select a key from the list, even if there is only one */
 			if (select_key_cb != NULL) {
@@ -1132,3 +1181,38 @@ get_utf8_locale(int category)
     return localebuf;
 }
 #endif
+
+/*
+ * Note: this function is a hack to detect if the gpg engine in use support the '--export-filter' options 'keep-uid=...' and
+ * 'drop-subkey=...' (since 2.2.9) needed for exporting a minimal Autocrypt key.
+ */
+static void
+gpg_check_capas(const gchar *gpg_path, const gchar *version)
+{
+	gchar *gpg_args[] = { (gchar *) gpg_path, "--export", "--export-filter", "keep-uid=primary=1", "0000000000000000", NULL };
+	gint exit_status;
+	guint major;
+	guint minor;
+	guint release;
+
+	gpg_capas.gpg_path = g_strdup(gpg_path);
+
+	/* check for the "--export-filter keep-uid=..." option */
+	if (g_spawn_sync(NULL, gpg_args, NULL, G_SPAWN_STDOUT_TO_DEV_NULL + G_SPAWN_STDERR_TO_DEV_NULL, NULL, NULL, NULL, NULL,
+					 &exit_status, NULL)) {
+#if       GLIB_CHECK_VERSION(2, 70, 0)
+		gpg_capas.export_filter_uid = g_spawn_check_wait_status(exit_status, NULL);
+#else  /* GLIB_CHECK_VERSION(2, 70, 0) */
+		gpg_capas.export_filter_uid = g_spawn_check_exit_status(exit_status, NULL);
+#endif /* GLIB_CHECK_VERSION(2, 70, 0) */
+	}
+	g_debug("%s supports '--export-filter keep-uid=...': %d", gpg_path, gpg_capas.export_filter_uid);
+
+	/* check for the "--export-filter drop-subkey=usage!~e && usage!~s" option */
+	if (sscanf(version, "%u.%u.%u", &major, &minor, &release) == 3) {
+		gpg_capas.export_filter_subkey = (major > 2U) ||
+			((major == 2U) && (minor > 2U)) ||
+			((major == 2U) && (minor == 2U) && (release >= 9U));
+	}
+	g_debug("%s supports '--export-filter drop-subkey=...': %d", gpg_path, gpg_capas.export_filter_subkey);
+}
