@@ -43,9 +43,14 @@
 #include <gtksourceview/gtksource.h>
 #endif
 
+#if HAVE_CANBERRA
+#include <canberra-gtk.h>
+#endif
+
 #include "misc.h"
 #include "missing.h"
 #include "x509-cert-widget.h"
+#include "html.h"
 #include <glib/gi18n.h>
 
 static GThread *main_thread_id;
@@ -86,6 +91,9 @@ libbalsa_init(void)
 #if HAVE_OSMO
     LIBBALSA_TYPE_ADDRESS_BOOK_OSMO;
 #endif
+#ifdef HAVE_HTML_WIDGET
+    libbalsa_html_init();
+#endif
 }
 
 
@@ -691,6 +699,40 @@ libbalsa_source_view_new(gboolean highlight_phrases)
 }
 #endif  /* HAVE_GTKSOURCEVIEW */
 
+#if HAVE_CANBERRA
+gboolean
+libbalsa_play_sound_event(const gchar *event_id, GError **error)
+{
+	GdkScreen *screen;
+	gint rc;
+
+	g_return_val_if_fail(event_id != NULL, FALSE);
+
+	screen = gdk_screen_get_default();
+	rc = ca_context_play(ca_gtk_context_get_for_screen(screen), 0, CA_PROP_EVENT_ID, event_id, NULL);
+	if (rc != 0) {
+		g_set_error(error, LIBBALSA_ERROR_QUARK, rc, _("Cannot play sound event “%s”: %s"), event_id, ca_strerror(rc));
+	}
+	return rc == 0;
+}
+
+gboolean
+libbalsa_play_sound_file(const gchar *filename, GError **error)
+{
+	GdkScreen *screen;
+	gint rc;
+
+	g_return_val_if_fail(filename != NULL, FALSE);
+
+	screen = gdk_screen_get_default();
+	rc = ca_context_play(ca_gtk_context_get_for_screen(screen), 0, CA_PROP_MEDIA_FILENAME, filename, NULL);
+	if (rc != 0) {
+		g_set_error(error, LIBBALSA_ERROR_QUARK, rc, _("Cannot play sound file “%s”: %s"), filename, ca_strerror(rc));
+	}
+	return rc == 0;
+}
+#endif /* HAVE_CANBERRA */
+
 /*
  * Error domains for GError:
  */
