@@ -26,6 +26,7 @@
    - Lack of inline functions in C increases program complexity. This cost
    can be accepted.
    - thorough analysis of memory usage is needed.
+   2022-03-21: All code conditional on MESSAGE_COPY_CONTENT is removed.
  */
 
 #if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
@@ -61,18 +62,11 @@ struct _LibBalsaMessage {
     /* the mailbox this message belongs to */
     LibBalsaMailbox *mailbox;
 
-    /* flags */
-    LibBalsaMessageFlag flags;
-
     /* headers */
     LibBalsaMessageHeaders *headers;
-    int updated; /** whether complete headers have been fetched */
 
     GMimeMessage *mime_msg;
 
-    /* sender address */
-    InternetAddressList *sender;
-
     /* subject line; we still need it here for sending;
      * although _SET_SUBJECT might resolve it(?)
      * but we can set to to NULL unless there is no mailbox, like
@@ -88,21 +82,9 @@ struct _LibBalsaMessage {
     /* message ID */
     gchar *message_id;
 
-    /* GPG sign and/or encrypt message (sending) */
-    guint gpg_mode;
-
-    /* attach the GnuPG public key to the message (sending) */
-    gboolean att_pubkey;
-
-    /* protection (i.e. sign/encrypt) status (received message) */
-    LibBalsaMsgProtectState prot_state;
-
     /* sender identity, required for choosing a forced GnuPG or S/MIME key */
     LibBalsaIdentity *ident;
 
-    /* request a DSN (sending) */
-    gboolean request_dsn;
-
     /* a forced multipart subtype or NULL for mixed; used only for
      * sending */
     gchar *subtype;
@@ -110,19 +92,40 @@ struct _LibBalsaMessage {
     /* additional message content type parameters; used only for sending */
     GList *parameters;
 
-    /* message body */
-    guint body_ref;
-    LibBalsaMessageBody *body_list;
-
-    glong msgno;     /* message no; always copy for faster sorting;
-                      * counting starts at 1. */
-#if MESSAGE_COPY_CONTENT
-    glong length;   /* byte len */
-#endif /* MESSAGE_COPY_CONTENT */
-
     gchar *tempdir;     /* to hold named parts */
 
+    /* message body */
+    LibBalsaMessageBody *body_list;
+
+    /* end of pointers, begin 64-bit ints */
+    gint64 length;   /* byte len */
+
+    /* end of 64-bit ints, begin ints */
+    /* GnuPG or S/MIME sign and/or encrypt message (sending), or status of received message */
+    guint crypt_mode;
+
+    guint body_ref;
+
+    /* flags */
+    LibBalsaMessageFlag flags;
+
+    guint msgno;     /* message no; always copy for faster sorting;
+                      * counting starts at 1. */
+
+    /* GPG sign and/or encrypt message (sending) */
+    guint gpg_mode;
+
+    /* end of ints, begin bit fields */
     unsigned has_all_headers : 1;
+
+    /* Indicate that uid's should always be trusted when signing a message */
+    unsigned always_trust : 1;
+
+    /* attach the GnuPG public key to the message (sending) */
+    unsigned att_pubkey : 1;
+
+    /* request a DSN (sending) */
+    unsigned request_dsn : 1;
 };
 
 G_DEFINE_TYPE(LibBalsaMessage,
@@ -133,20 +136,25 @@ static void
 libbalsa_message_init(LibBalsaMessage * message)
 {
     message->headers = g_new0(LibBalsaMessageHeaders, 1);
-    message->flags = 0;
     message->mailbox = NULL;
-    message->sender = NULL;
     message->subj = NULL;
     message->references = NULL;
     message->in_reply_to = NULL;
     message->message_id = NULL;
-    message->subtype = 0;
+    message->subtype = NULL;
     message->parameters = NULL;
     message->body_ref = 0;
     message->body_list = NULL;
     message->has_all_headers = 0;
-    message->prot_state = LIBBALSA_MSG_PROTECT_NONE;
+    message->crypt_mode = LIBBALSA_PROTECT_NONE;
     message->ident = NULL;
+    message->body_list = NULL;
+    message->body_ref = 0;
+    message->flags = LIBBALSA_MESSAGE_FLAG_NONE;
+    message->gpg_mode = 0;
+    message->has_all_headers = 0;
+    message->att_pubkey = 0;
+    message->request_dsn = 0;
 }
 
 
@@ -178,7 +186,6 @@ libbalsa_message_dispose(GObject * object)
 {
     LibBalsaMessage *message = LIBBALSA_MESSAGE(object);
 
-    g_clear_object(&message->sender);
     g_clear_object(&message->mime_msg);
     g_clear_object(&message->ident);
 
@@ -199,9 +206,7 @@ libbalsa_message_finalize(GObject * object)
 
     g_free(message->message_id);
     g_free(message->subtype);
-#if MESSAGE_COPY_CONTENT
     g_free(message->subj);
-#endif /* MESSAGE_COPY_CONTENT */
 
     libbalsa_message_body_free(message->body_list);
 
@@ -244,6 +249,11 @@ libbalsa_message_headers_destroy(LibBalsaMessageHeaders *headers)
 	headers->from = NULL;
     }
 
+    if (headers->sender) {
+	g_object_unref(headers->sender);
+	headers->sender = NULL;
+    }
+
     if (headers->to_list) {
 	g_object_unref(headers->to_list);
 	headers->to_list = NULL;
@@ -488,7 +498,7 @@ prepend_header_misc(GList      *res,
 {
     char lcname[28]; /* one byte longer than the longest ignored header */
     static const char ignored_headers[] =
-        "subject date from to cc bcc "
+        "subject date from sender to cc bcc "
         "message-id references in-reply-to status lines"
         "disposition-notification-to";
     unsigned i;
@@ -644,19 +654,19 @@ libbalsa_message_get_attach_icon(LibBalsaMessage * message)
 
     if (libbalsa_message_is_encrypted(message)) {
 	attach_icon = LIBBALSA_MESSAGE_ATTACH_ENCR;
-    } else if (message->prot_state != LIBBALSA_MSG_PROTECT_NONE ||
+    } else if (message->crypt_mode != LIBBALSA_PROTECT_NONE ||
 	libbalsa_message_is_signed(message)) {
-	switch (message->prot_state) {
-	case LIBBALSA_MSG_PROTECT_SIGN_GOOD:
+	switch (message->crypt_mode) {
+	case LIBBALSA_PROTECT_SIGN_GOOD:
 	    attach_icon = LIBBALSA_MESSAGE_ATTACH_GOOD;
             break;
-	case LIBBALSA_MSG_PROTECT_SIGN_NOTRUST:
+	case LIBBALSA_PROTECT_SIGN_NOTRUST:
 	    attach_icon = LIBBALSA_MESSAGE_ATTACH_NOTRUST;
             break;
-	case LIBBALSA_MSG_PROTECT_SIGN_BAD:
+	case LIBBALSA_PROTECT_SIGN_BAD:
 	    attach_icon = LIBBALSA_MESSAGE_ATTACH_BAD;
             break;
-	case LIBBALSA_MSG_PROTECT_CRYPT:
+	case LIBBALSA_PROTECT_ENCRYPT:
 	    attach_icon = LIBBALSA_MESSAGE_ATTACH_ENCR;
             break;
 	default:
@@ -712,7 +722,6 @@ libbalsa_message_reply(LibBalsaMessage *message)
  */
 gboolean
 libbalsa_message_body_ref(LibBalsaMessage *message,
-                          gboolean         read,
                           gboolean         fetch_all_headers)
 {
     LibBalsaFetchFlag flags = 0;
@@ -726,7 +735,7 @@ libbalsa_message_body_ref(LibBalsaMessage *message,
 
     libbalsa_lock_mailbox(message->mailbox);
 
-    if (fetch_all_headers && !message->has_all_headers)
+    if (fetch_all_headers && message->has_all_headers == 0)
         flags |= LB_FETCH_RFC822_HEADERS;
 
     if ((message->body_ref == 0) && !message->body_list) {
@@ -934,65 +943,12 @@ libbalsa_message_set_dispnotify(LibBalsaMessage * message,
 /* libbalsa_message_get_subject:
    get constant pointer to the subject of the message; 
 */
-#ifdef MESSAGE_COPY_CONTENT
 const gchar *
 libbalsa_message_get_subject(LibBalsaMessage *message)
 {
     return message->subj != NULL ? message->subj : _("(No subject)");
 }
 
-
-#else /* MESSAGE_COPY_CONTENT */
-const gchar *
-libbalsa_message_get_subject(LibBalsaMessage* msg)
-{
-    const gchar *ret;
-    if(msg->subj == NULL &&
-       msg->mime_msg != NULL && msg->mailbox != NULL) { /* a message in a mailbox... */
-        g_return_val_if_fail(MAILBOX_OPEN(msg->mailbox), NULL);
-        ret = g_mime_message_get_subject(msg->mime_msg);
-        libbalsa_message_set_subject_from_header(msg, ret);
-    } else
-	ret = msg->subj;
-
-    return ret ? ret : _("(No subject)");
-}
-
-
-guint
-libbalsa_message_get_lines(LibBalsaMessage* msg)
-{
-    /* set the line count */
-    const char *value;
-    if (msg->mime_msg == NULL)
-	return 0;
-    value = g_mime_object_get_header(msg->mime_msg, "Lines");
-    if (value == NULL)
-	return 0;
-    return atoi(value);
-}
-glong
-libbalsa_message_get_length(LibBalsaMessage* msg)
-{
-    /* set the length */
-    const char *value;
-    if (msg->mime_msg == NULL)
-	return 0;
-    value = g_mime_object_get_header(msg->mime_msg, "Content-Length");
-    if (value == NULL)
-	return 0;
-    return atoi(value);
-}
-
-glong
-libbalsa_message_get_no(LibBalsaMessage* msg)
-{
-    return msg->msgno;
-}
-
-
-#endif /* MESSAGE_COPY_CONTENT */
-
 /* Populate headers from mime_msg, but only the members that are needed
  * all the time. */
 
@@ -1022,6 +978,11 @@ lb_message_headers_basic_from_gmime(LibBalsaMessageHeaders *headers,
             lb_message_address_list_ref(g_mime_message_get_from(mime_msg));
     }
 
+    if (headers->sender == NULL) {
+        headers->sender =
+            lb_message_address_list_ref(g_mime_message_get_addresses(mime_msg, GMIME_ADDRESS_TYPE_SENDER));
+    }
+
     if (headers->date == 0) {
         GDateTime *datetime;
 
@@ -1030,23 +991,14 @@ lb_message_headers_basic_from_gmime(LibBalsaMessageHeaders *headers,
             headers->date = g_date_time_to_unix(datetime);
     }
 
-    if (headers->to_list == NULL)
+    if (headers->to_list == NULL) {
         headers->to_list =
             lb_message_recipients(mime_msg, GMIME_ADDRESS_TYPE_TO);
+    }
 
     if (headers->content_type == NULL) {
-        /* If we could:
-         * headers->content_type =
-         *     g_mime_content_type_copy
-         *         (g_mime_object_get_content_type(mime_msg->mime_part));
-         */
-        GMimeContentType *content_type;
-        gchar *str;
-
-        content_type = g_mime_object_get_content_type(mime_msg->mime_part);
-        str = g_mime_content_type_get_mime_type(content_type);
-        headers->content_type = g_mime_content_type_parse(libbalsa_parser_options(), str);
-        g_free(str);
+        g_set_object(&headers->content_type,
+                     g_mime_object_get_content_type(mime_msg->mime_part));
     }
 }
 
@@ -1123,15 +1075,20 @@ libbalsa_message_init_from_gmime(LibBalsaMessage * message,
     g_return_if_fail(LIBBALSA_IS_MESSAGE(message));
     g_return_if_fail(GMIME_IS_MESSAGE(mime_msg));
 
-#ifdef MESSAGE_COPY_CONTENT
     header = g_mime_message_get_subject(mime_msg);
     libbalsa_message_set_subject_from_header(message, header);
 
     header = g_mime_object_get_header(GMIME_OBJECT(mime_msg), "Content-Length");
-    if (header)
-        message->length = atoi(header);
+    if (header) {
+        char *endptr;
+
+        message->length = strtoll(header, &endptr, 10);
+        if (*endptr != '\0') {
+            message->length = -1;
+            g_debug("Bad Content-Length header: “%s”; using -1", header);
+        }
+    }
 
-#endif /* MESSAGE_COPY_CONTENT */
     header = g_mime_message_get_message_id(mime_msg);
     if (header)
         message->message_id = g_strdup(header);
@@ -1269,9 +1226,7 @@ lbmsg_set_header(LibBalsaMessage *message,
             g_free(val);
             return FALSE;
         }
-#if MESSAGE_COPY_CONTENT
         libbalsa_message_set_subject_from_header(message, value);
-#endif /* MESSAGE_COPY_CONTENT */
     } else if (g_ascii_strcasecmp(name, "Date") == 0) {
         GDateTime *datetime;
 
@@ -1299,13 +1254,15 @@ lbmsg_set_header(LibBalsaMessage *message,
     } else if ((headers->dispnotify_to == NULL) &&
                (g_ascii_strcasecmp(name, "Disposition-Notification-To") == 0)) {
         headers->dispnotify_to = internet_address_list_parse(libbalsa_parser_options(), value);
-    } else
-#ifdef MESSAGE_COPY_CONTENT
-    if (g_ascii_strcasecmp(name, "Content-Length") == 0) {
-        message->length = atoi(value);
-    } else
-#endif /* MESSAGE_COPY_CONTENT */
-    if (all) {
+    } else if (g_ascii_strcasecmp(name, "Content-Length") == 0) {
+        char *endptr;
+
+        message->length = strtoll(value, &endptr, 10);
+        if (*endptr != '\0') {
+            message->length = -1;
+            g_debug("Bad Content-Length header: “%s”; using -1", value);
+        }
+    } else if (all) {
         headers->user_hdrs =
             g_list_prepend(headers->user_hdrs,
                            libbalsa_create_hdr_pair(name, g_strdup(value)));
@@ -1594,7 +1551,7 @@ libbalsa_message_get_message_id(LibBalsaMessage *message)
 }
 
 
-glong
+guint
 libbalsa_message_get_msgno(LibBalsaMessage *message)
 {
     g_return_val_if_fail(LIBBALSA_IS_MESSAGE(message), 0);
@@ -1603,7 +1560,7 @@ libbalsa_message_get_msgno(LibBalsaMessage *message)
 }
 
 
-glong
+gint64
 libbalsa_message_get_length(LibBalsaMessage *message)
 {
     g_return_val_if_fail(LIBBALSA_IS_MESSAGE(message), 0);
@@ -1617,16 +1574,7 @@ libbalsa_message_get_has_all_headers(LibBalsaMessage *message)
 {
     g_return_val_if_fail(LIBBALSA_IS_MESSAGE(message), FALSE);
 
-    return message->has_all_headers;
-}
-
-
-InternetAddressList *
-libbalsa_message_get_sender(LibBalsaMessage *message)
-{
-    g_return_val_if_fail(LIBBALSA_IS_MESSAGE(message), 0);
-
-    return message->sender;
+    return message->has_all_headers != 0;
 }
 
 
@@ -1635,7 +1583,7 @@ libbalsa_message_get_request_dsn(LibBalsaMessage *message)
 {
     g_return_val_if_fail(LIBBALSA_IS_MESSAGE(message), FALSE);
 
-    return message->request_dsn;
+    return message->request_dsn != 0;
 }
 
 
@@ -1677,11 +1625,20 @@ libbalsa_message_get_subtype(LibBalsaMessage *message)
 
 
 guint
-libbalsa_message_get_gpg_mode(LibBalsaMessage *message)
+libbalsa_message_get_crypt_mode(LibBalsaMessage *message)
 {
     g_return_val_if_fail(LIBBALSA_IS_MESSAGE(message), 0);
 
-    return message->gpg_mode;
+    return message->crypt_mode;
+}
+
+
+gboolean
+libbalsa_message_get_always_trust(LibBalsaMessage *message)
+{
+    g_return_val_if_fail(LIBBALSA_IS_MESSAGE(message), FALSE);
+
+    return message->always_trust != 0;
 }
 
 
@@ -1699,16 +1656,7 @@ libbalsa_message_get_attach_pubkey(LibBalsaMessage *message)
 {
     g_return_val_if_fail(LIBBALSA_IS_MESSAGE(message), FALSE);
 
-    return message->att_pubkey;
-}
-
-
-LibBalsaMsgProtectState
-libbalsa_message_get_protect_state(LibBalsaMessage *message)
-{
-    g_return_val_if_fail(LIBBALSA_IS_MESSAGE(message), 0);
-
-    return message->prot_state;
+    return message->att_pubkey != 0;
 }
 
 
@@ -1757,7 +1705,7 @@ libbalsa_message_set_mailbox(LibBalsaMessage *message,
 
 void
 libbalsa_message_set_msgno(LibBalsaMessage *message,
-                           glong            msgno)
+                           guint            msgno)
 {
     g_return_if_fail(LIBBALSA_IS_MESSAGE(message));
 
@@ -1771,14 +1719,13 @@ libbalsa_message_set_has_all_headers(LibBalsaMessage *message,
 {
     g_return_if_fail(LIBBALSA_IS_MESSAGE(message));
 
-    message->has_all_headers = has_all_headers;
+    message->has_all_headers = has_all_headers ? 1 : 0;
 }
 
 
-#if MESSAGE_COPY_CONTENT
 void
 libbalsa_message_set_length(LibBalsaMessage *message,
-                            glong            length)
+                            gint64           length)
 {
     g_return_if_fail(LIBBALSA_IS_MESSAGE(message));
 
@@ -1786,8 +1733,6 @@ libbalsa_message_set_length(LibBalsaMessage *message,
 }
 
 
-#endif /* MESSAGE_COPY_CONTENT */
-
 void
 libbalsa_message_set_mime_message(LibBalsaMessage *message,
                               GMimeMessage    *mime_message)
@@ -1799,17 +1744,6 @@ libbalsa_message_set_mime_message(LibBalsaMessage *message,
 }
 
 
-void
-libbalsa_message_set_sender(LibBalsaMessage     *message,
-                            InternetAddressList *sender)
-{
-    g_return_if_fail(LIBBALSA_IS_MESSAGE(message));
-    g_return_if_fail(sender == NULL || IS_INTERNET_ADDRESS_LIST(sender));
-
-    g_set_object(&message->sender, sender);
-}
-
-
 void
 libbalsa_message_set_message_id(LibBalsaMessage *message,
                                 const gchar     *message_id)
@@ -1821,23 +1755,13 @@ libbalsa_message_set_message_id(LibBalsaMessage *message,
 }
 
 
-void
-libbalsa_message_set_protect_state(LibBalsaMessage        *message,
-                                LibBalsaMsgProtectState prot_state)
-{
-    g_return_if_fail(LIBBALSA_IS_MESSAGE(message));
-
-    message->prot_state = prot_state;
-}
-
-
 void
 libbalsa_message_set_request_dsn(LibBalsaMessage *message,
                                  gboolean         request_dsn)
 {
     g_return_if_fail(LIBBALSA_IS_MESSAGE(message));
 
-    message->request_dsn = request_dsn;
+    message->request_dsn = request_dsn ? 1 : 0;
 }
 
 
@@ -1887,22 +1811,32 @@ libbalsa_message_set_in_reply_to(LibBalsaMessage *message,
 
 
 void
-libbalsa_message_set_gpg_mode(LibBalsaMessage *message,
-                              guint            mode)
+libbalsa_message_set_crypt_mode(LibBalsaMessage *message,
+                                guint            mode)
 {
     g_return_if_fail(LIBBALSA_IS_MESSAGE(message));
 
-    message->gpg_mode = mode;
+    message->crypt_mode = mode;
+}
+
+
+void
+libbalsa_message_set_always_trust(LibBalsaMessage *message,
+                                  gboolean         mode)
+{
+    g_return_if_fail(LIBBALSA_IS_MESSAGE(message));
+
+    message->always_trust = mode ? 1 : 0;
 }
 
 
 void
 libbalsa_message_set_attach_pubkey(LibBalsaMessage *message,
-                                gboolean         att_pubkey)
+                                   gboolean         att_pubkey)
 {
     g_return_if_fail(LIBBALSA_IS_MESSAGE(message));
 
-    message->att_pubkey = att_pubkey;
+    message->att_pubkey = att_pubkey ? 1 : 0;
 }
 
 
