@@ -76,7 +76,7 @@ process_mime_part(LibBalsaMessage * message, LibBalsaMessageBody * body,
 	html_type = libbalsa_html_type(mime_type);
 	g_free(mime_type);
 
-	if (ignore_html && html_type)
+	if (ignore_html && (html_type != LIBBALSA_HTML_TYPE_NONE))
 	    break;
 
 #ifdef HAVE_HTML_WIDGET
@@ -183,7 +183,7 @@ content2reply(LibBalsaMessageBody * root, gchar * reply_prefix_str,
     LibBalsaMessageBody *body;
     GString *reply = NULL, *res;
 
-    libbalsa_message_body_ref(message, FALSE, FALSE);
+    libbalsa_message_body_ref(message, FALSE);
     for (body = root; body; body = body->next) {
 	res = process_mime_part(message, body, reply_prefix_str, llen,
                                 ignore_html, flow);
@@ -471,144 +471,6 @@ libbalsa_wrap_rfc2646(gchar * par, gint width, gboolean from_screen,
     return g_string_free(result, FALSE);
 }
 
-/*
- * libbalsa_wrap_view(GtkTextView * view, gint length)
- *
- * Wrap the text in a GtkTextView to the given line length.
- */
-
-/* Forward references: */
-static GtkTextTag *get_quote_tag(GtkTextIter * iter);
-static gint get_quote_depth(GtkTextIter * iter, gchar ** quote_string);
-static gchar *get_line(GtkTextBuffer * buffer, GtkTextIter * iter);
-static gboolean is_in_url(GtkTextIter * iter, gint offset,
-                          GtkTextTag * url_tag);
-
-void
-libbalsa_wrap_view(GtkTextView * view, gint length)
-{
-    GtkTextBuffer *buffer = gtk_text_view_get_buffer(view);
-    GtkTextTagTable *table = gtk_text_buffer_get_tag_table(buffer);
-    GtkTextTag *url_tag = gtk_text_tag_table_lookup(table, "url");
-    GtkTextIter iter;
-    PangoContext *context = gtk_widget_get_pango_context(GTK_WIDGET(view));
-    PangoLanguage *language = pango_context_get_language(context);
-    PangoLogAttr *log_attrs = NULL;
-
-    gtk_text_buffer_get_start_iter(buffer, &iter);
-
-    /* Loop over original lines. */
-    while (!gtk_text_iter_is_end(&iter)) {
-	GtkTextTag *quote_tag;
-	gchar *quote_string;
-	gint quote_len;
-
-	quote_tag = get_quote_tag(&iter);
-	quote_len = get_quote_depth(&iter, &quote_string);
-	if (quote_string) {
-	    if (quote_string[quote_len])
-		quote_len++;
-	    else {
-		gchar *tmp = g_strconcat(quote_string, " ", NULL);
-		g_free(quote_string);
-		quote_string = tmp;
-	    }
-	}
-
-	/* Loop over breaks within the original line. */
-	while (!gtk_text_iter_ends_line(&iter)) {
-	    gchar *line;
-	    gint num_chars;
-	    gint attrs_len;
-	    gint offset;
-	    gint len;
-	    gint brk_offset = 0;
-	    gunichar c = 0;
-	    gboolean in_space = FALSE;
-
-	    line = get_line(buffer, &iter);
-	    num_chars = g_utf8_strlen(line, -1);
-	    attrs_len = num_chars + 1;
-	    log_attrs = g_renew(PangoLogAttr, log_attrs, attrs_len);
-	    pango_get_log_attrs(line, -1, -1, language, log_attrs, attrs_len);
-	    g_free(line);
-
-	    for (len = offset = quote_len;
-		 len < length && offset < num_chars; offset++) {
-		gtk_text_iter_set_line_offset(&iter, offset);
-		c = gtk_text_iter_get_char(&iter);
-		if (c == '\t')
-		    len = ((len + 8) / 8) * 8;
-		else if (g_unichar_isprint(c))
-		    len++;
-
-		if (log_attrs[offset].is_line_break
-		    && !is_in_url(&iter, offset, url_tag))
-		    brk_offset = offset;
-	    }
-
-	    if (len < length)
-		break;
-
-	    in_space = g_unichar_isspace(c);
-	    if (brk_offset > quote_len && !in_space)
-		/* Break at the last break we saw. */
-		gtk_text_iter_set_line_offset(&iter, brk_offset);
-	    else {
-                GtkTextIter start = iter;
-		/* Break at the next line break. */
-		if (offset <= quote_len)
-		    offset = quote_len + 1;
-		while (offset < num_chars
-		       && (is_in_url(&iter, offset, url_tag)
-			   || !log_attrs[offset].is_line_break))
-		    offset++;
-
-		if (offset >= num_chars)
-		    /* No next line break. */
-		    break;
-
-                /* Trim extra trailing whitespace */
-                gtk_text_iter_forward_char(&start);
-                gtk_text_buffer_delete(buffer, &start, &iter);
-	    }
-
-	    gtk_text_buffer_insert(buffer, &iter, "\n", 1);
-	    if (quote_string)
-		gtk_text_buffer_insert_with_tags(buffer, &iter,
-						 quote_string, -1,
-						 quote_tag, NULL);
-	}
-	g_free(quote_string);
-	gtk_text_iter_forward_line(&iter);
-    }
-    g_free(log_attrs);
-}
-
-/* Find the quote tag, if any, at iter; doesn't move iter; returns tag
- * or NULL. */
-static GtkTextTag *
-get_quote_tag(GtkTextIter * iter)
-{
-    GtkTextTag *quote_tag = NULL;
-    GSList *list;
-    GSList *tag_list = gtk_text_iter_get_tags(iter);
-
-    for (list = tag_list; list; list = list->next) {
-        GtkTextTag *tag = list->data;
-        gchar *name;
-        g_object_get(tag, "name", &name, NULL);
-        if (name) {
-            if (!strncmp(name, "quote-", 6))
-                quote_tag = tag_list->data;
-            g_free(name);
-        }
-    }
-    g_slist_free(tag_list);
-
-    return quote_tag;
-}
-
 /* Move the iter over a string of consecutive '>' characters, and an
  * optional ' ' character; returns the number of '>'s, and, if
  * quote_string is not NULL, stores an allocated copy of the string
@@ -653,16 +515,6 @@ get_line(GtkTextBuffer * buffer, GtkTextIter * iter)
     return gtk_text_buffer_get_slice(buffer, iter, &end, TRUE);
 }
 
-/* Move the iter to position offset in the current line, and test
- * whether it's in an URL. */
-static gboolean
-is_in_url(GtkTextIter * iter, gint offset, GtkTextTag * url_tag)
-{
-    gtk_text_iter_set_line_offset(iter, offset);
-    return url_tag ? (gtk_text_iter_has_tag(iter, url_tag)
-                      && !gtk_text_iter_starts_tag(iter, url_tag)) : FALSE;
-}
-
 /* Remove soft newlines and associated quote strings from num_paras
  * paragraphs in the buffer, starting at the line before iter; if
  * num_paras < 0, process the whole buffer. */
@@ -854,7 +706,9 @@ get_url_reg(void)
         NULL,
         "(((https?|ftps?|nntp)://)|(mailto:|news:))"
             "(%[0-9A-F]{2}|[-_.!~*';/?:@&=+$,#[:alnum:]])+"
-            "(%[0-9A-F]{2}|[-_!~*';/?:@&=+$,#[:alnum:]])",
+            /* do not include a trailing period or comma as part of the match;
+             * it is more likely to be punctuation than part of a URL */
+            "(%[0-9A-F]{2}|[-_!~*';/?:@&=+$#[:alnum:]])",
         __func__,
         "url regex compilation failed"
     };
@@ -949,6 +803,7 @@ libbalsa_insert_with_url(GtkTextBuffer * buffer,
     match = g_regex_match(url_reg, chars, 0, &url_match)
         && g_match_info_fetch_pos(url_match, 0, &start_pos, &end_pos)
         && chars + start_pos < line_end;
+    g_match_info_free(url_match);
 
     while (match) {
         gchar *spc;
@@ -1025,15 +880,15 @@ libbalsa_insert_with_url(GtkTextBuffer * buffer,
 
         chars += end_pos;
         if (prescanner(chars, line_end - chars)) {
-            g_match_info_free(url_match);
             match = g_regex_match(url_reg, chars, 0, &url_match)
                 && g_match_info_fetch_pos(url_match, 0, &start_pos,
                                           &end_pos)
                 && chars + start_pos < line_end;
-        } else
+            g_match_info_free(url_match);
+        } else {
             match = FALSE;
+        }
     }
-    g_match_info_free(url_match);
 
     gtk_text_buffer_insert_with_tags(buffer, &iter, chars,
                                      line_end - chars, tag, NULL);
@@ -1299,3 +1154,97 @@ libbalsa_text_to_html(const gchar * title, const gchar * body, const gchar * lan
     /* return the utf-8 encoded text/html */
     return g_string_free(html_body, FALSE);
 }
+
+/*
+ * libbalsa_wrap_quoted string
+ * Wraps the string, prefixing wrapped lines with any quote string
+ * Uses the same wrapping strategy as libbalsa_wrap_string()
+ * Returns a newly allocated string--deallocate with g_free() when done
+*/
+char *
+libbalsa_wrap_quoted_string(const char *str,
+                            unsigned    width,
+                            GRegex     *quote_regex)
+{
+    char **lines;
+    char **line;
+    GString *wrapped;
+    PangoLogAttr *log_attrs = NULL;
+
+    g_return_val_if_fail(str != NULL, NULL);
+    g_return_val_if_fail(quote_regex != NULL, NULL);
+
+    lines = g_strsplit(str, "\n", -1);
+    wrapped = g_string_new(NULL);
+
+    for (line = lines; *line != NULL; line++) {
+        unsigned quote_len, quote_len_utf8;
+        const char *start_ptr, *break_ptr, *ptr;
+        const unsigned minl = width / 2;
+        unsigned ptr_offset, start_offset, break_offset;
+        int num_chars;
+        int attrs_len;
+        unsigned cursor;
+
+        num_chars = g_utf8_strlen(*line, -1);
+        attrs_len = num_chars + 1;
+        log_attrs = g_renew(PangoLogAttr, log_attrs, attrs_len);
+        pango_get_log_attrs(*line, -1, -1, pango_language_get_default(), log_attrs, attrs_len);
+
+        libbalsa_match_regex(*line, quote_regex, NULL, &quote_len);
+
+        g_string_append_len(wrapped, *line, quote_len);
+        ptr = *line + quote_len;
+
+        ptr_offset = g_utf8_pointer_to_offset(*line, ptr);
+        cursor = quote_len_utf8 = ptr_offset;
+
+        start_ptr = break_ptr = ptr;
+        start_offset = break_offset = ptr_offset;
+
+        while (*ptr != '\0') {
+            gunichar c = g_utf8_get_char(ptr);
+
+            if (c == '\t')
+                cursor += 8 - cursor % 8;
+            else
+                cursor++;
+
+            if (log_attrs[ptr_offset].is_line_break) {
+                break_ptr = ptr;
+                break_offset = ptr_offset;
+            }
+
+            if (cursor >= width && break_offset >= start_offset + minl && !g_unichar_isspace(c)) {
+                const char *end_ptr, *test_ptr;
+                gunichar test_char;
+
+                /* Back up over whitespace */
+                test_ptr = break_ptr;
+                do {
+                    end_ptr = test_ptr;
+                    test_ptr = g_utf8_prev_char(test_ptr);
+                    test_char = g_utf8_get_char(test_ptr);
+                } while (test_ptr > start_ptr && g_unichar_isspace(test_char));
+
+                g_string_append_len(wrapped, start_ptr, end_ptr - start_ptr);
+                g_string_append_c(wrapped, '\n');
+                g_string_append_len(wrapped, *line, quote_len);
+
+                start_ptr = break_ptr;
+                start_offset = break_offset;
+                cursor = quote_len_utf8 + ptr_offset - start_offset;
+            }
+            ptr = g_utf8_next_char(ptr);
+            ptr_offset++;
+        }
+
+        g_string_append(wrapped, start_ptr);
+        g_string_append_c(wrapped, '\n');
+    }
+
+    g_free(log_attrs);
+    g_strfreev(lines);
+
+    return (char *) g_string_free(wrapped, FALSE);
+}
