@@ -222,10 +222,11 @@ static GMimePart *lb_create_pubkey_part(LibBalsaMessage  *message,
 										GError          **error);
 
 static gpointer balsa_send_message_real(SendMessageInfo *info);
-static LibBalsaMsgCreateResult libbalsa_message_create_mime_message(LibBalsaMessage *message,
-                                                                    gboolean         flow,
-                                                                    gboolean         postponing,
-                                                                    GError         **error);
+static LibBalsaMsgCreateResult create_mime_message(LibBalsaMessage *message,
+                                                   gboolean         flow,
+                                                   gboolean         postponing,
+                                                   GMimeMessage   **return_message,
+                                                   GError         **error);
 static LibBalsaMsgCreateResult libbalsa_create_msg(LibBalsaMessage *message,
                                                    gboolean         flow,
                                                    GError         **error);
@@ -682,7 +683,7 @@ lbs_process_queue_msg(guint 		   msgno,
 	}
 
 	/* check the smtp server */
-	libbalsa_message_body_ref(msg, TRUE, TRUE);
+	libbalsa_message_body_ref(msg, TRUE);
 	smtp_server_name = libbalsa_message_get_user_header(msg, "X-Balsa-SmtpServer");
 	if (!smtp_server_name) {
 		smtp_server_name = libbalsa_smtp_server_get_name(NULL);
@@ -982,8 +983,9 @@ balsa_send_message_error(MessageQueueItem *mqi,
 			libbalsa_message_change_flags(mqi->orig, LIBBALSA_MESSAGE_FLAG_FLAGGED, LIBBALSA_MESSAGE_FLAG_DELETED);
 		}
 	}
-	libbalsa_information(LIBBALSA_INFORMATION_ERROR, _("Sending message failed: %s\nMessage left in your outbox."),
-		error->message);
+	libbalsa_information(LIBBALSA_INFORMATION_ERROR,
+		_("Sending message to SMTP server %s failed: %s\nMessage left in your outbox."),
+		libbalsa_smtp_server_get_name(mqi->smsg_info->smtp_server), error->message);
 }
 
 static void
@@ -1004,6 +1006,7 @@ balsa_send_message_syslog(const gchar            *smtp_server,
 		syslog(LOG_MAIL | LOG_NOTICE, "%s Error='%s'", syslog_msg->str,
 				(server_error != NULL) ? server_error->message : "unknown");
 	}
+        g_string_free(syslog_msg, TRUE);
 }
 
 static gpointer
@@ -1183,10 +1186,11 @@ ia_list_not_empty(InternetAddressList *ial)
 }
 
 static LibBalsaMsgCreateResult
-libbalsa_message_create_mime_message(LibBalsaMessage *message,
-                                     gboolean         flow,
-                                     gboolean         postponing,
-                                     GError         **error)
+create_mime_message(LibBalsaMessage *message,
+                    gboolean         flow,
+                    gboolean         postponing,
+                    GMimeMessage   **return_message,
+                    GError         **error)
 {
     GMimeObject *mime_root = NULL;
     GMimeMessage *mime_message;
@@ -1204,7 +1208,7 @@ libbalsa_message_create_mime_message(LibBalsaMessage *message,
     /* attach the public key only if we send the message, not if we just postpone it */
     if (!postponing &&
         libbalsa_message_get_attach_pubkey(message) &&
-        ((libbalsa_message_get_gpg_mode(message) & LIBBALSA_PROTECT_PROTOCOL) != 0)) {
+        ((libbalsa_message_get_crypt_mode(message) & LIBBALSA_PROTECT_PROTOCOL) != 0)) {
     	attach_pubkey = TRUE;
     }
 
@@ -1359,7 +1363,7 @@ libbalsa_message_create_mime_message(LibBalsaMessage *message,
             /* in '2440 mode, touch *only* the first body! */
             if (!postponing &&
                 (body == libbalsa_message_get_body_list(body->message)) &&
-                ((gpg_mode = libbalsa_message_get_gpg_mode(message)) > 0) &&
+                ((gpg_mode = libbalsa_message_get_crypt_mode(message)) != 0) &&
                 ((gpg_mode & LIBBALSA_PROTECT_OPENPGP) != 0)) {
                 use_gpg_mode = gpg_mode;
             } else {
@@ -1510,6 +1514,10 @@ libbalsa_message_create_mime_message(LibBalsaMessage *message,
 #endif
 
     libbalsa_message_set_mime_message(message, mime_message);
+    /* message now holds a reference to mime_message, so we do not add one here: */
+    if (return_message != NULL)
+        *return_message = mime_message;
+
     g_object_unref(mime_message);
 
     return LIBBALSA_MESSAGE_CREATE_OK;
@@ -1525,25 +1533,26 @@ libbalsa_message_create_mime_message(LibBalsaMessage *message,
 gboolean
 libbalsa_message_postpone(LibBalsaMessage *message,
                           LibBalsaMailbox *draftbox,
-                          LibBalsaMessage *reply_message,
                           gchar          **extra_headers,
                           gboolean         flow,
                           GError         **error)
 {
-    if (!libbalsa_message_get_mime_message(message)
-        && (libbalsa_message_create_mime_message(message, flow,
-                                                 TRUE, error) !=
-            LIBBALSA_MESSAGE_CREATE_OK)) {
-        return FALSE;
+    GMimeMessage *mime_message;
+
+    mime_message = libbalsa_message_get_mime_message(message);
+    if (mime_message == NULL) {
+        LibBalsaMsgCreateResult res =
+            create_mime_message(message, flow, TRUE, &mime_message, error);
+        if (res != LIBBALSA_MESSAGE_CREATE_OK)
+            return FALSE;
     }
 
     if (extra_headers != NULL) {
+        GMimeObject *object = GMIME_OBJECT(mime_message);
         gint i;
 
-        for (i = 0; extra_headers[i] && extra_headers[i + 1]; i += 2) {
-            g_mime_object_set_header(GMIME_OBJECT(libbalsa_message_get_mime_message(message)),
-                                     extra_headers[i], extra_headers[i + 1], NULL);
-        }
+        for (i = 0; extra_headers[i] && extra_headers[i + 1]; i += 2)
+            g_mime_object_set_header(object, extra_headers[i], extra_headers[i + 1], NULL);
     }
 
     return libbalsa_message_copy(message, draftbox, error);
@@ -1642,16 +1651,17 @@ libbalsa_create_msg(LibBalsaMessage *message,
                     gboolean         flow,
                     GError         **error)
 {
-    if (libbalsa_message_get_mime_message(message) == NULL) {
+    GMimeMessage *mime_message;
+
+    mime_message = libbalsa_message_get_mime_message(message);
+    if (mime_message == NULL) {
         LibBalsaMsgCreateResult res =
-            libbalsa_message_create_mime_message(message, flow,
-                                                 FALSE, error);
-        if (res != LIBBALSA_MESSAGE_CREATE_OK) {
+            create_mime_message(message, flow, FALSE, &mime_message, error);
+        if (res != LIBBALSA_MESSAGE_CREATE_OK)
             return res;
-        }
     }
 
-    libbalsa_set_message_id(libbalsa_message_get_mime_message(message));
+    libbalsa_set_message_id(mime_message);
 
     return LIBBALSA_MESSAGE_CREATE_OK;
 }
@@ -1792,8 +1802,8 @@ libbalsa_create_rfc2440_buffer(LibBalsaMessage *message,
                                GtkWindow       *parent,
                                GError         **error)
 {
-    gint mode = libbalsa_message_get_gpg_mode(message);
-    gboolean always_trust = (mode & LIBBALSA_PROTECT_ALWAYS_TRUST) != 0;
+    guint mode = libbalsa_message_get_crypt_mode(message);
+    gboolean always_trust = libbalsa_message_get_always_trust(message);
 
     switch (mode & LIBBALSA_PROTECT_MODE) {
     case LIBBALSA_PROTECT_SIGN:       /* sign only */
@@ -1874,7 +1884,7 @@ do_multipart_crypto(LibBalsaMessage *message,
     gpgme_protocol_t protocol;
     gboolean always_trust;
 
-    mode = libbalsa_message_get_gpg_mode(message);
+    mode = libbalsa_message_get_crypt_mode(message);
 
     /* check if we shall do any protection */
     if ((mode & LIBBALSA_PROTECT_MODE) == 0) {
@@ -1884,14 +1894,14 @@ do_multipart_crypto(LibBalsaMessage *message,
     /* check which protocol should be used */
     if (mode & LIBBALSA_PROTECT_RFC3156) {
         protocol = GPGME_PROTOCOL_OpenPGP;
-    } else if (mode & LIBBALSA_PROTECT_SMIMEV3) {
+    } else if (mode & LIBBALSA_PROTECT_SMIME) {
         protocol = GPGME_PROTOCOL_CMS;
     } else if (mode & LIBBALSA_PROTECT_OPENPGP) {
         return LIBBALSA_MESSAGE_CREATE_OK;  /* already done... */
     } else {
         return LIBBALSA_MESSAGE_ENCRYPT_ERROR;  /* hmmm.... */
     }
-    always_trust = (mode & LIBBALSA_PROTECT_ALWAYS_TRUST) != 0;
+    always_trust = libbalsa_message_get_always_trust(message);
     /* sign and/or encrypt */
     switch (mode & LIBBALSA_PROTECT_MODE) {
     case LIBBALSA_PROTECT_SIGN:       /* sign message */
@@ -1931,7 +1941,7 @@ do_multipart_crypto(LibBalsaMessage *message,
                              (headers->bcc_list)));
         }
 
-        if ((libbalsa_message_get_gpg_mode(message) & LIBBALSA_PROTECT_SIGN) != 0) {
+        if ((libbalsa_message_get_crypt_mode(message) & LIBBALSA_PROTECT_SIGN) != 0) {
             success =
                 libbalsa_sign_encrypt_mime_object(mime_root,
                                                   lb_send_from(message, protocol),
