@@ -1063,38 +1063,6 @@ libbalsa_dialog_flags(void)
 
 #define BALSA_AB_NOTIFICATION "balsa-ab-notification"
 
-static void
-balsa_ab_notification_notify_cb(GNotification *notification, GApplication *application)
-{
-    gboolean send;
-
-    send = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(notification), "send"));
-    if (send) {
-        g_application_send_notification(application,
-                                        BALSA_AB_NOTIFICATION, notification);
-    } else {
-        g_application_withdraw_notification(application, BALSA_AB_NOTIFICATION);
-    }
-}
-
-static void
-balsa_ab_notification_shutdown_cb(GApplication *application, GNotification *notification)
-{
-    g_object_unref(notification);
-}
-
-static void
-balsa_ab_setup_libbalsa_notification(GApplication *application)
-{
-    GNotification *notification;
-
-    notification = libbalsa_notification_new("BalsaAb");
-    g_signal_connect(notification, "notify",
-                     G_CALLBACK(balsa_ab_notification_notify_cb), application);
-    g_signal_connect(application, "shutdown",
-                     G_CALLBACK(balsa_ab_notification_shutdown_cb), notification);
-}
-
 int
 main(int argc, char *argv[])
 {
@@ -1104,8 +1072,13 @@ main(int argc, char *argv[])
     GList *l;
     GError *error = NULL;
 
+#if GLIB_CHECK_VERSION(2, 73, 3)
+    application =
+        gtk_application_new("org.desktop.BalsaAb", G_APPLICATION_DEFAULT_FLAGS);
+#else  /* GLIB_CHECK_VERSION(2, 73, 3) */
     application =
         gtk_application_new("org.desktop.BalsaAb", G_APPLICATION_FLAGS_NONE);
+#endif /* GLIB_CHECK_VERSION(2, 73, 3) */
     if (!g_application_register(G_APPLICATION(application), NULL, &error)) {
         g_warning("Could not register address book editor: %s", error->message);
         g_error_free(error);
@@ -1124,7 +1097,8 @@ main(int argc, char *argv[])
 #endif
 
     bab_init();
-    balsa_ab_setup_libbalsa_notification((GApplication *) application);
+    libbalsa_information_init(G_APPLICATION(application),
+    	_("Balsa Address Book"), BALSA_AB_NOTIFICATION);
     g_mime_init();
     libbalsa_parser_options_init();
 
