@@ -412,9 +412,15 @@ balsa_app_init(void)
     balsa_app.error_message = 0;
     balsa_app.debug_message = 0;
 
-    balsa_app.notify_new_mail_sound = 1;
+#ifdef HAVE_CANBERRA
+    balsa_app.notify_new_mail_sound = 0;
+#endif
+
     balsa_app.notify_new_mail_dialog = 0;
-    balsa_app.notify_new_mail_icon = 1;
+
+#ifdef ENABLE_SYSTRAY
+    balsa_app.enable_systray_icon = 0;
+#endif
 
     /* Local and IMAP */
     balsa_app.local_scan_depth = 1;
@@ -940,3 +946,20 @@ balsa_quote_regex_new(void)
 
     return g_regex_ref(regex);
 }
+
+gboolean
+balsa_autocrypt_in_use(void)
+{
+	gboolean result = FALSE;
+#ifdef ENABLE_AUTOCRYPT
+	GList *ident;
+
+	for (ident = balsa_app.identities; !result && (ident != NULL); ident = ident->next) {
+                LibBalsaIdentity *identity = LIBBALSA_IDENTITY(ident->data);
+                AutocryptMode autocrypt_mode = libbalsa_identity_get_autocrypt_mode(identity);
+		result = autocrypt_mode != AUTOCRYPT_DISABLE;
+	}
+#endif  /* ENABLE_AUTOCRYPT */
+	return result;
+}
+
