@@ -150,6 +150,7 @@ struct _BalsaMimeWidgetText {
     GList               *cite_bar_list;
     gint                 cite_bar_dimension;
     gint                 phrase_hl;
+    GtkTextTag          *invisible;
 };
 
 G_DEFINE_TYPE(BalsaMimeWidgetText, balsa_mime_widget_text, BALSA_TYPE_MIME_WIDGET)
@@ -237,7 +238,7 @@ balsa_mime_widget_new_text(BalsaMessage * bm, LibBalsaMessageBody * mime_body,
 
     /* handle HTML if possible */
     html_type = libbalsa_html_type(content_type);
-    if (html_type) {
+    if (html_type != LIBBALSA_HTML_TYPE_NONE) {
         BalsaMimeWidget *html_widget = NULL;
 
 #ifdef HAVE_HTML_WIDGET
@@ -251,7 +252,7 @@ balsa_mime_widget_new_text(BalsaMessage * bm, LibBalsaMessageBody * mime_body,
     if (alloced < 0) {
         balsa_information(LIBBALSA_INFORMATION_ERROR,
                           _("Could not save a text part: %s"),
-                          err ? err->message : "Unknown error");
+                          err ? err->message : _("Unknown error"));
         g_clear_error(&err);
         return NULL;
     }
@@ -294,8 +295,13 @@ balsa_mime_widget_new_text(BalsaMessage * bm, LibBalsaMessageBody * mime_body,
 #if HAVE_GTKSOURCEVIEW
 	       && !GTK_SOURCE_IS_VIEW(widget)
 #endif
-	       )
-	libbalsa_wrap_string(ptr, balsa_app.browse_wrap_length);
+	       ) {
+        GRegex *rex = balsa_quote_regex_new();
+        char *wrapped = libbalsa_wrap_quoted_string(ptr, balsa_app.browse_wrap_length, rex);
+        g_regex_unref(rex);
+        g_free(ptr);
+        ptr = wrapped;
+    }
 
     fill_text_buf_cited(mwt, widget, ptr,
                         libbalsa_message_body_is_flowed(mime_body),
@@ -548,7 +554,7 @@ url_copy_cb(GtkWidget * menu_item, message_url_t * uri)
 
     display = gtk_widget_get_display(menu_item);
     clipboard =
-        gtk_clipboard_get_for_display(display, GDK_SELECTION_PRIMARY);
+        gtk_clipboard_get_for_display(display, GDK_SELECTION_CLIPBOARD);
     gtk_clipboard_set_text(clipboard, uri->url, -1);
 }
 
@@ -977,8 +983,7 @@ draw_cite_bar_real(gpointer data, gpointer user_data)
     GtkTextView * view;
     GtkTextBuffer * buffer;
     gint dimension;
-    GdkRectangle location;
-    gint x_pos;
+    gint buffer_y;
     gint y_pos;
     gint height;
 
@@ -997,14 +1002,14 @@ draw_cite_bar_real(gpointer data, gpointer user_data)
     }
 
     /* get the locations */
-    gtk_text_view_get_iter_location(view, &bar->start_iter, &location);
+    gtk_text_view_get_line_yrange(view, &bar->start_iter, &buffer_y, NULL);
     gtk_text_view_buffer_to_window_coords(view, GTK_TEXT_WINDOW_TEXT,
-                                          location.x, location.y,
-                                          &x_pos, &y_pos);
-    gtk_text_view_get_iter_location(view, &bar->end_iter, &location);
+                                          0, buffer_y,
+                                          NULL, &y_pos);
+    gtk_text_view_get_line_yrange(view, &bar->end_iter, &buffer_y, NULL);
     gtk_text_view_buffer_to_window_coords(view, GTK_TEXT_WINDOW_TEXT,
-                                          location.x, location.y,
-                                          &x_pos, &height);
+                                          0, buffer_y,
+                                          NULL, &height);
     height -= y_pos;
 
     /* add a new widget if necessary */
@@ -1112,10 +1117,10 @@ bmwt_html_prefer_html_changed(GtkCheckMenuItem *checkmenuitem,
 }
 
 static void
-bmwt_html_load_images_changed(GtkCheckMenuItem *checkmenuitem,
+bmwt_html_load_external_content_changed(GtkCheckMenuItem *checkmenuitem,
                               gpointer          user_data)
 {
-	libbalsa_html_prefer_set_load_images(INTERNET_ADDRESS_LIST(user_data),
+	libbalsa_html_prefer_set_load_content(INTERNET_ADDRESS_LIST(user_data),
 		gtk_check_menu_item_get_active(checkmenuitem));
 }
 
@@ -1187,25 +1192,56 @@ bmwt_html_populate_popup_menu(BalsaMessage * bm,
     g_signal_connect(menuitem, "toggled", G_CALLBACK(bmwt_html_prefer_html_changed), from);
     gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
 
-    menuitem = gtk_check_menu_item_new_with_label(_("Load images for this sender"));
+    menuitem = gtk_check_menu_item_new_with_label(_("Load external content for this sender"));
     gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menuitem),
-        libbalsa_html_get_load_images(from));
+        libbalsa_html_get_load_content(from));
     gtk_widget_set_sensitive(menuitem, from != NULL);
-    g_signal_connect(menuitem, "toggled", G_CALLBACK(bmwt_html_load_images_changed), from);
+    g_signal_connect(menuitem, "toggled", G_CALLBACK(bmwt_html_load_external_content_changed), from);
     gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
 
     gtk_widget_show_all(GTK_WIDGET(menu));
 }
 
+static void
+bmwt_html_populate_url_menu(GtkMenu *menu, const gchar *url)
+{
+	GtkWidget *menuitem;
+	message_url_t *popup_url;
+
+	popup_url = g_new0(message_url_t, 1U);
+	popup_url->url = g_strdup(url);
+	g_object_set_data_full(G_OBJECT(menu), "url", popup_url, (GDestroyNotify) free_url);
+
+	menuitem = gtk_menu_item_new_with_label(_("Copy link"));
+	g_signal_connect(menuitem, "activate", G_CALLBACK(url_copy_cb), popup_url);
+	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
+
+	menuitem = gtk_menu_item_new_with_label(_("Open link"));
+	g_signal_connect(menuitem, "activate", G_CALLBACK(url_open_cb), popup_url);
+	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
+
+	menuitem = gtk_menu_item_new_with_label(_("Send link…"));
+	g_signal_connect(menuitem, "activate", G_CALLBACK(url_send_cb), popup_url);
+	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
+
+	gtk_widget_show_all(GTK_WIDGET(menu));
+}
+
 static gboolean
 bmwt_html_popup_context_menu(GtkWidget * html, BalsaMessage * bm)
 {
     GtkWidget *menu;
+    const gchar *popup_url;
     const GdkEvent *event;
     GdkEvent *current_event = NULL;
 
     menu = gtk_menu_new();
-    bmwt_html_populate_popup_menu(bm, html, GTK_MENU(menu));
+    popup_url = (const gchar *) g_object_get_data(G_OBJECT(html), LIBBALSA_HTML_POPUP_URL);
+    if (popup_url != NULL) {
+    	bmwt_html_populate_url_menu(GTK_MENU(menu), popup_url);
+    } else {
+    	bmwt_html_populate_popup_menu(bm, html, GTK_MENU(menu));
+    }
 
     /* In WebKit2, the context menu signal is asynchronous, so the
      * GdkEvent is no longer current; instead it is preserved and passed
@@ -1276,7 +1312,7 @@ bm_widget_new_html(BalsaMessage * bm, LibBalsaMessageBody * mime_body)
         libbalsa_html_new(mime_body,
                          (LibBalsaHtmlCallback) bm_widget_on_url,
                          (LibBalsaHtmlCallback) handle_url,
-                         libbalsa_html_get_load_images(from));
+                         libbalsa_html_get_load_content(from));
     gtk_container_add(GTK_CONTAINER(mwt), widget);
 
     g_object_set_data(G_OBJECT(widget), "mime-body", mime_body);
@@ -1411,7 +1447,6 @@ fill_text_buf_cited(BalsaMimeWidgetText *mwt,
     GdkScreen *screen;
     GtkTextBuffer *buffer;
     GdkRGBA *rgba;
-    GtkTextTag *invisible;
     LibBalsaUrlInsertInfo url_info;
     guint cite_level;
     guint cite_start;
@@ -1440,11 +1475,11 @@ fill_text_buf_cited(BalsaMimeWidgetText *mwt,
                                NULL);
 
     if (rex != NULL) {
-        invisible = gtk_text_buffer_create_tag(buffer, "hide-cite",
-                                               "size-points", (gdouble) 0.01,
-                                               NULL);
+        mwt->invisible = gtk_text_buffer_create_tag(buffer, "hide-cite",
+                                                    "size-points", (gdouble) 0.0,
+                                                    NULL);
     } else {
-        invisible = NULL;
+        mwt->invisible = NULL;
     }
 
     mwt->url_list = NULL;
@@ -1497,7 +1532,7 @@ fill_text_buf_cited(BalsaMimeWidgetText *mwt,
                 gtk_text_buffer_insert_with_tags(buffer, &cite_iter,
                                                  text_body,
                                                  cite_idx,
-                                                 tag, invisible, NULL);
+                                                 tag, mwt->invisible, NULL);
                 text_body += cite_idx;
 
                 /* append a zero-width space if the remainder of the line is
