@@ -14,7 +14,7 @@
  * GNU General Public License for more details.
  *  
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  */
 
 /*
@@ -835,9 +835,6 @@ condition_dialog_response(GtkWidget * dialog, gint response,
 {
     LibBalsaCondition *new_cnd;
     GError *err = NULL;
-#if !GTK_CHECK_VERSION(3, 22, 0)
-    GdkScreen *screen;
-#endif /* GTK_CHECK_VERSION(3, 22, 0) */
 
     switch (response) {
     case GTK_RESPONSE_OK:       /* OK button */
@@ -893,15 +890,9 @@ condition_dialog_response(GtkWidget * dialog, gint response,
         gtk_widget_hide(dialog);
         break;
     case GTK_RESPONSE_HELP:     /* Help button */
-#if GTK_CHECK_VERSION(3, 22, 0)
         gtk_show_uri_on_window(GTK_WINDOW(dialog),
                                "help:balsa/win-filters#win-condition",
                                gtk_get_current_event_time(), &err);
-#else  /* GTK_CHECK_VERSION(3, 22, 0) */
-        screen = gtk_widget_get_screen(dialog);
-        gtk_show_uri(screen, "help:balsa/win-filters#win-condition",
-                     gtk_get_current_event_time(), &err);
-#endif  /* GTK_CHECK_VERSION(3, 22, 0) */
 	if (err) {
 	    balsa_information_parented(GTK_WINDOW(dialog),
 		    LIBBALSA_INFORMATION_WARNING,
@@ -924,7 +915,7 @@ add_button(GtkWidget *grid, const char *label, int col,
     GtkWidget* button = gtk_button_new_with_mnemonic(label);
     gtk_widget_set_hexpand(button, TRUE);
     gtk_grid_attach(GTK_GRID(grid), button, col, 4, 1, 1);
-    g_signal_connect(G_OBJECT(button), "clicked",
+    g_signal_connect(button, "clicked",
                      action,
                      GINT_TO_POINTER(p));
 }
@@ -934,7 +925,7 @@ add_check(GtkWidget *grid, const char *label, int col, int row)
     GtkWidget* res = gtk_check_button_new_with_mnemonic(label);
     gtk_widget_set_hexpand(res, TRUE);
     gtk_grid_attach(GTK_GRID(grid), res, col, row, 1, 1);
-    g_signal_connect(G_OBJECT(res), "toggled",
+    g_signal_connect(res, "toggled",
                      G_CALLBACK(fe_condition_changed_cb), NULL);
     return res;
 }
@@ -969,14 +960,14 @@ get_field_frame(void)
         gtk_check_button_new_with_mnemonic(_("_User header:"));
     gtk_widget_set_hexpand(fe_matching_fields_us_head, TRUE);
     gtk_grid_attach(GTK_GRID(grid), fe_matching_fields_us_head, 0, 3, 1, 1);
-    g_signal_connect(G_OBJECT(fe_matching_fields_us_head), "toggled",
+    g_signal_connect(fe_matching_fields_us_head, "toggled",
                      G_CALLBACK(fe_match_field_user_header_cb), NULL);
 
     fe_user_header = gtk_combo_box_text_new_with_entry();
     for (list = fe_user_headers_list; list; list = list->next)
         gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(fe_user_header),
                                        list->data);
-    g_signal_connect(G_OBJECT(fe_user_header), "changed",
+    g_signal_connect(fe_user_header, "changed",
                      G_CALLBACK(fe_condition_changed_cb), NULL);
     gtk_widget_set_hexpand(fe_user_header, TRUE);
     gtk_grid_attach(GTK_GRID(grid), fe_user_header, 1, 3, 1, 1);
@@ -1020,7 +1011,7 @@ build_type_notebook()
     gtk_widget_set_hexpand(fe_type_simple_label, TRUE);
     gtk_grid_attach(GTK_GRID(grid), fe_type_simple_label, 0, 0, 1, 1);
     fe_type_simple_entry = gtk_entry_new();
-    g_signal_connect(G_OBJECT(fe_type_simple_entry), "changed",
+    g_signal_connect(fe_type_simple_entry, "changed",
                      G_CALLBACK(fe_condition_changed_cb), NULL);
     gtk_widget_set_hexpand(fe_type_simple_entry, TRUE);
     gtk_grid_attach(GTK_GRID(grid), fe_type_simple_entry, 0, 1, 1, 1);
@@ -1028,7 +1019,7 @@ build_type_notebook()
 				  fe_type_simple_entry);
 
     button = gtk_button_new_with_mnemonic(_("Contain/Does _Not Contain"));
-    g_signal_connect(G_OBJECT(button), "clicked",
+    g_signal_connect(button, "clicked",
                      G_CALLBACK(fe_negate_condition), NULL);
     gtk_widget_set_hexpand(button, TRUE);
     gtk_grid_attach(GTK_GRID(grid), button, 0, 2, 1, 1);
@@ -1076,15 +1067,15 @@ build_type_notebook()
                      GTK_FILL | GTK_SHRINK | GTK_EXPAND, GTK_SHRINK, 2, 2);
     button = gtk_button_new_with_mnemonic(_("A_dd"));
     gtk_box_pack_start(GTK_BOX(box), button, TRUE, TRUE, 0);
-    g_signal_connect(G_OBJECT(button), "clicked",
+    g_signal_connect(button, "clicked",
                      G_CALLBACK(fe_add_pressed), NULL);
     fe_regex_remove_button = gtk_button_new_with_mnemonic(_("_Remove"));
     gtk_box_pack_start(GTK_BOX(box), fe_regex_remove_button, TRUE, TRUE, 0);
-    g_signal_connect(G_OBJECT(fe_regex_remove_button), "clicked",
+    g_signal_connect(fe_regex_remove_button, "clicked",
                      G_CALLBACK(fe_remove_pressed), NULL);
     button = gtk_button_new_with_mnemonic(_("One _Matches/None Matches"));
     gtk_box_pack_start(GTK_BOX(box), button, TRUE, TRUE, 0);
-    g_signal_connect(G_OBJECT(button), "clicked",
+    g_signal_connect(button, "clicked",
                      G_CALLBACK(fe_negate_condition), NULL);
 
     fe_type_regex_entry = gtk_entry_new();
@@ -1112,18 +1103,18 @@ build_type_notebook()
     gtk_grid_attach(GTK_GRID(page), fe_type_date_label, 0, 0, 2, 1);
     fe_type_date_low_entry = gtk_entry_new();
     gtk_widget_set_hexpand(fe_type_date_low_entry, TRUE);
-    g_signal_connect(G_OBJECT(fe_type_date_low_entry), "changed",
+    g_signal_connect(fe_type_date_low_entry, "changed",
                      G_CALLBACK(fe_condition_changed_cb), NULL);
     gtk_grid_attach(GTK_GRID(page), fe_type_date_low_entry, 0, 1, 1, 1);
     fe_type_date_high_entry = gtk_entry_new();
     gtk_widget_set_hexpand(fe_type_date_high_entry, TRUE);
-    g_signal_connect(G_OBJECT(fe_type_date_high_entry), "changed",
+    g_signal_connect(fe_type_date_high_entry, "changed",
                      G_CALLBACK(fe_condition_changed_cb), NULL);
     gtk_grid_attach(GTK_GRID(page), fe_type_date_high_entry, 1, 1, 1, 1);
 
     button = gtk_button_new_with_label(_("Inside/outside the date interval"));
     gtk_widget_set_hexpand(button, TRUE);
-    g_signal_connect(G_OBJECT(button), "clicked",
+    g_signal_connect(button, "clicked",
                      G_CALLBACK(fe_negate_condition), NULL);
     gtk_grid_attach(GTK_GRID(page), button, 0, 2, 2, 1);
 
@@ -1164,7 +1155,7 @@ build_type_notebook()
         }
     button = gtk_button_new_with_label(_("Match when one flag is set/when no flag is set"));
     gtk_widget_set_hexpand(button, TRUE);
-    g_signal_connect(G_OBJECT(button), "clicked",
+    g_signal_connect(button, "clicked",
                      G_CALLBACK(fe_negate_condition), NULL);
     gtk_grid_attach(GTK_GRID(page),
                      button,
@@ -1184,7 +1175,7 @@ void build_condition_dialog(GtkWidget * condition_dialog)
     label = gtk_label_new_with_mnemonic(_("Search T_ype:"));
     gtk_box_pack_start(GTK_BOX(box), label, FALSE, FALSE, 5);
     fe_search_option_menu = 
-        fe_build_option_menu(fe_search_type, ELEMENTS(fe_search_type),
+        fe_build_option_menu(fe_search_type, G_N_ELEMENTS(fe_search_type),
                              G_CALLBACK(fe_typesmenu_cb), field_frame);
     gtk_box_pack_start(GTK_BOX(box), fe_search_option_menu, FALSE, FALSE, 5);
     gtk_label_set_mnemonic_widget(GTK_LABEL(label), fe_search_option_menu);
@@ -1251,9 +1242,9 @@ fe_edit_condition(GtkWidget * throwaway,gpointer is_new_cnd)
 	libbalsa_macosx_menu_for_parent(condition_dialog, GTK_WINDOW(fe_window));
 #endif
         g_object_add_weak_pointer(G_OBJECT(condition_dialog),
-                                  (gpointer) &condition_dialog);
+                                  (gpointer *) &condition_dialog);
 
-        g_signal_connect(G_OBJECT(condition_dialog), "response",
+        g_signal_connect(condition_dialog, "response",
                          G_CALLBACK(condition_dialog_response), NULL);
         /* Now we build the dialog*/
         build_condition_dialog(condition_dialog);
@@ -1327,25 +1318,30 @@ update_filters_mailbox(GtkTreeModel * model, GtkTreePath * path,
 {
     BalsaMailboxNode *mbnode;
     LibBalsaMailbox *mailbox;
+    const gchar *mailbox_url;
+    const gchar *mailbox_name;
     gchar *tmp;
 
     gtk_tree_model_get(model, iter, 0, &mbnode, -1);
-    mailbox = mbnode->mailbox;
+    mailbox = balsa_mailbox_node_get_mailbox(mbnode);
     g_object_unref(mbnode);
     if (!mailbox)
 	return FALSE;
 
     /* First we free the filters list (which is now obsolete) */
-    g_slist_foreach(mailbox->filters, (GFunc) g_free, NULL);
-    g_slist_free(mailbox->filters);
-    mailbox->filters = NULL;
+    libbalsa_mailbox_set_filters(mailbox, NULL);
+
     /* Second we replace old filters name by the new ones
      * Note : deleted filters are also removed */
     if (!filters_names_changes)
 	return FALSE;
-    tmp = mailbox_filters_section_lookup(mailbox->url ?
-					 mailbox->url : mailbox->name);
-    if (tmp) {
+
+    mailbox_url = libbalsa_mailbox_get_url(mailbox);
+    mailbox_name = libbalsa_mailbox_get_name(mailbox);
+    tmp = mailbox_filters_section_lookup(mailbox_url ?
+					 mailbox_url : mailbox_name);
+
+    if (tmp != NULL) {
 	gchar **filters_names = NULL;
 	gboolean def;
 	gint nb_filters;
@@ -1439,8 +1435,7 @@ void fe_destroy_window_cb(GtkWidget * widget,gpointer throwaway)
     new_filters_names=NULL;
 
     /* free all strings in fe_user_headers_list */
-    g_list_foreach(fe_user_headers_list,(GFunc)g_free,NULL);
-    g_list_free(fe_user_headers_list);
+    g_list_free_full(fe_user_headers_list, g_free);
     fe_user_headers_list = NULL;
 
     fe_already_open=FALSE;
@@ -1460,9 +1455,6 @@ fe_dialog_response(GtkWidget * dialog, gint response, gpointer data)
     GtkTreeIter iter;
     gboolean valid;
     GError *err = NULL;
-#if !GTK_CHECK_VERSION(3, 22, 0)
-    GdkScreen *screen;
-#endif /* GTK_CHECK_VERSION(3, 22, 0) */
 
     switch (response) {
     case GTK_RESPONSE_OK:       /* OK button */
@@ -1501,14 +1493,8 @@ fe_dialog_response(GtkWidget * dialog, gint response, gpointer data)
         break;
 
     case GTK_RESPONSE_HELP:     /* Help button */
-#if GTK_CHECK_VERSION(3, 22, 0)
         gtk_show_uri_on_window(GTK_WINDOW(dialog), "help:balsa/win-filters",
                                    gtk_get_current_event_time(), &err);
-#else  /* GTK_CHECK_VERSION(3, 22, 0) */
-        screen = gtk_widget_get_screen(dialog);
-        gtk_show_uri(screen, "help:balsa/win-filters",
-                     gtk_get_current_event_time(), &err);
-#endif  /* GTK_CHECK_VERSION(3, 22, 0) */
 	if (err) {
 	    balsa_information_parented(GTK_WINDOW(dialog),
 		    LIBBALSA_INFORMATION_WARNING,
