@@ -14,7 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  */
 
 #if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
@@ -36,12 +36,6 @@
 #  include "macosx-helpers.h"
 #endif
 
-#ifndef MAX
-#define MAX(a, b) ((a) > (b) ? (a) : (b))
-#endif
-
-/* Uncomment the next line to check for invalid action names */
-/* #define BALSA_TOOLBAR_DEBUG_ACTIONS */
 
 /* Enumeration for GtkTreeModel columns. */
 enum {
@@ -94,11 +88,7 @@ static GtkWidget *create_toolbar_page(BalsaToolbarModel * model,
 static GtkWidget *tp_list_new(void);
 static gboolean tp_list_iter_is_first(GtkWidget * list, GtkTreeIter * iter);
 static gboolean tp_list_iter_is_last(GtkWidget * list, GtkTreeIter * iter);
-#ifndef BALSA_TOOLBAR_DEBUG_ACTIONS
 static void tp_page_refresh_available(ToolbarPage * page);
-#else /* BALSA_TOOLBAR_DEBUG_ACTIONS */
-static void tp_page_refresh_available(ToolbarPage * page, GActionMap * map);
-#endif /* BALSA_TOOLBAR_DEBUG_ACTIONS */
 static void tp_page_refresh_current(ToolbarPage * page);
 static void tp_page_refresh_preview(ToolbarPage * page);
 static void tp_page_swap_rows(ToolbarPage * page, gboolean forward);
@@ -106,7 +96,6 @@ static void tp_page_add_selected(ToolbarPage * page);
 static void tp_page_remove_selected(ToolbarPage * page);
 static void tp_store_set(GtkListStore * store, GtkTreeIter * iter,
                          gint item);
-static void replace_nl_with_space(char* str);
 
 /* Public methods. */
 
@@ -128,7 +117,8 @@ customize_dialog_cb(GtkWidget * widget, gpointer data)
 
     /* There can only be one */
     if (customize_widget) {
-        gtk_window_present(GTK_WINDOW(customize_widget));
+        gtk_window_present_with_time(GTK_WINDOW(customize_widget),
+                                     gtk_get_current_event_time());
         return;
     }
 
@@ -144,8 +134,8 @@ customize_dialog_cb(GtkWidget * widget, gpointer data)
     libbalsa_macosx_menu_for_parent(customize_widget, GTK_WINDOW(active_window));
 #endif
     g_object_add_weak_pointer(G_OBJECT(customize_widget),
-                              (gpointer) & customize_widget);
-    g_signal_connect(G_OBJECT(customize_widget), "response",
+                              (gpointer *) & customize_widget);
+    g_signal_connect(customize_widget, "response",
                      G_CALLBACK(tp_dialog_response_cb), NULL);
 
     notebook = gtk_notebook_new();
@@ -161,9 +151,7 @@ customize_dialog_cb(GtkWidget * widget, gpointer data)
     model = balsa_window_get_toolbar_model();
     group = g_simple_action_group_new();
     balsa_window_add_action_entries(G_ACTION_MAP(group));
-#ifdef BALSA_TOOLBAR_DEBUG_ACTIONS
-    g_print("main window\n");
-#endif /* BALSA_TOOLBAR_DEBUG_ACTIONS */
+    g_debug("%s: main window", __func__);
     child = create_toolbar_page(model, G_ACTION_MAP(group));
     g_object_unref(group);
     gtk_notebook_append_page(GTK_NOTEBOOK(notebook), child,
@@ -172,9 +160,7 @@ customize_dialog_cb(GtkWidget * widget, gpointer data)
     model = sendmsg_window_get_toolbar_model();
     group = g_simple_action_group_new();
     sendmsg_window_add_action_entries(G_ACTION_MAP(group));
-#ifdef BALSA_TOOLBAR_DEBUG_ACTIONS
-    g_print("compose window\n");
-#endif /* BALSA_TOOLBAR_DEBUG_ACTIONS */
+    g_debug("%s: compose window", __func__);
     child = create_toolbar_page(model, G_ACTION_MAP(group));
     g_object_unref(group);
     gtk_notebook_append_page(GTK_NOTEBOOK(notebook), child,
@@ -183,9 +169,7 @@ customize_dialog_cb(GtkWidget * widget, gpointer data)
     model = message_window_get_toolbar_model();
     group = g_simple_action_group_new();
     message_window_add_action_entries(G_ACTION_MAP(group));
-#ifdef BALSA_TOOLBAR_DEBUG_ACTIONS
-    g_print("message window\n");
-#endif /* BALSA_TOOLBAR_DEBUG_ACTIONS */
+    g_debug("%s: message window", __func__);
     child = create_toolbar_page(model, G_ACTION_MAP(group));
     g_object_unref(group);
     gtk_notebook_append_page(GTK_NOTEBOOK(notebook), child,
@@ -203,7 +187,7 @@ customize_dialog_cb(GtkWidget * widget, gpointer data)
         gtk_check_button_new_with_mnemonic(_("_Wrap button labels"));
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wrap_button),
                                  balsa_app.toolbar_wrap_button_text);
-    g_signal_connect(G_OBJECT(wrap_button), "toggled",
+    g_signal_connect(wrap_button, "toggled",
                      G_CALLBACK(wrap_toggled_cb), notebook);
     gtk_box_pack_start(GTK_BOX(option_box), wrap_button, FALSE, FALSE, 0);
 
@@ -291,11 +275,7 @@ standard_button_cb(GtkWidget *widget, ToolbarPage * page)
 {
     balsa_toolbar_model_clear(page->model);
     gtk_widget_set_sensitive(page->standard_button, FALSE);
-#ifndef BALSA_TOOLBAR_DEBUG_ACTIONS
     tp_page_refresh_available(page);
-#else /* BALSA_TOOLBAR_DEBUG_ACTIONS */
-    tp_page_refresh_available(page, NULL);
-#endif /* BALSA_TOOLBAR_DEBUG_ACTIONS */
     tp_page_refresh_current(page);
     balsa_toolbar_model_changed(page->model);
 }
@@ -362,9 +342,6 @@ current_selection_changed_cb(GtkTreeSelection * selection, ToolbarPage * page)
 static void
 tp_dialog_response_cb(GtkDialog * dialog, gint response, gpointer data)
 {
-#if !GTK_CHECK_VERSION(3, 22, 0)
-    GdkScreen *screen;
-#endif /* GTK_CHECK_VERSION(3, 22, 0) */
     GError *err = NULL;
 
     switch (response) {
@@ -373,14 +350,8 @@ tp_dialog_response_cb(GtkDialog * dialog, gint response, gpointer data)
         gtk_widget_destroy(GTK_WIDGET(dialog));
         break;
     case GTK_RESPONSE_HELP:
-#if GTK_CHECK_VERSION(3, 22, 0)
         gtk_show_uri_on_window(GTK_WINDOW(dialog), "help:balsa/toolbar-prefs",
                                gtk_get_current_event_time(), &err);
-#else  /* GTK_CHECK_VERSION(3, 22, 0) */
-        screen = gtk_widget_get_screen(GTK_WIDGET(dialog));
-        gtk_show_uri(screen, "help:balsa/toolbar-prefs",
-                     gtk_get_current_event_time(), &err);
-#endif /* GTK_CHECK_VERSION(3, 22, 0) */
         if (err) {
             balsa_information(LIBBALSA_INFORMATION_WARNING,
 		    _("Error displaying toolbar help: %s\n"),
@@ -541,30 +512,30 @@ create_toolbar_page(BalsaToolbarModel * model, GActionMap * map)
     gtk_container_add(GTK_CONTAINER(destination_scroll), page->current);
 
     /* UI signals */
-    g_signal_connect(G_OBJECT(page->available), "row-activated",
+    g_signal_connect(page->available, "row-activated",
                      G_CALLBACK(available_row_activated_cb), page);
     selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(page->available));
-    g_signal_connect(G_OBJECT(selection), "changed",
+    g_signal_connect(selection, "changed",
 		       G_CALLBACK(available_selection_changed_cb), page);
 
-    g_signal_connect(G_OBJECT(page->current), "row-activated",
+    g_signal_connect(page->current, "row-activated",
                      G_CALLBACK(current_row_activated_cb), page);
     selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(page->current));
-    g_signal_connect(G_OBJECT(selection), "changed",
+    g_signal_connect(selection, "changed",
 		       G_CALLBACK(current_selection_changed_cb), page);
 
-    g_signal_connect(G_OBJECT(page->add_button), "clicked",
+    g_signal_connect(page->add_button, "clicked",
 		       G_CALLBACK(add_button_cb), page);
-    g_signal_connect(G_OBJECT(page->remove_button), "clicked",
+    g_signal_connect(page->remove_button, "clicked",
 		       G_CALLBACK(remove_button_cb), page);
-    g_signal_connect(G_OBJECT(page->forward_button), "clicked",
+    g_signal_connect(page->forward_button, "clicked",
 		       G_CALLBACK(forward_button_cb), page);
-    g_signal_connect(G_OBJECT(page->back_button), "clicked",
+    g_signal_connect(page->back_button, "clicked",
 		       G_CALLBACK(back_button_cb), page);
 
-    g_signal_connect(G_OBJECT(page->standard_button), "clicked",
+    g_signal_connect(page->standard_button, "clicked",
 		       G_CALLBACK(standard_button_cb), page);
-    g_signal_connect(G_OBJECT(style_button), "clicked",
+    g_signal_connect(style_button, "clicked",
 		       G_CALLBACK(style_button_cb), page);
 
     gtk_widget_set_sensitive(page->add_button, FALSE);
@@ -574,11 +545,7 @@ create_toolbar_page(BalsaToolbarModel * model, GActionMap * map)
     gtk_widget_set_sensitive(page->standard_button,
                              !balsa_toolbar_model_is_standard(model));
 
-#ifndef BALSA_TOOLBAR_DEBUG_ACTIONS
     tp_page_refresh_available(page);
-#else /* BALSA_TOOLBAR_DEBUG_ACTIONS */
-    tp_page_refresh_available(page, map);
-#endif /* BALSA_TOOLBAR_DEBUG_ACTIONS */
     tp_page_refresh_current(page);
 
     return outer_box;
@@ -599,11 +566,7 @@ tp_find_icon(GArray * array, const gchar * icon)
 }
 
 static void
-#ifndef BALSA_TOOLBAR_DEBUG_ACTIONS
 tp_page_refresh_available(ToolbarPage * page)
-#else /* BALSA_TOOLBAR_DEBUG_ACTIONS */
-tp_page_refresh_available(ToolbarPage * page, GActionMap * map)
-#endif /* BALSA_TOOLBAR_DEBUG_ACTIONS */
 {
     GtkTreeSelection *selection =
         gtk_tree_view_get_selection(GTK_TREE_VIEW(page->available));
@@ -624,17 +587,6 @@ tp_page_refresh_available(ToolbarPage * page, GActionMap * map)
     gtk_list_store_clear(GTK_LIST_STORE(model));
 
     for (item = 0; item < toolbar_button_count; item++) {
-#ifdef BALSA_TOOLBAR_DEBUG_ACTIONS
-        if (map) {
-            if (item > 0) {
-                gchar *action;
-
-                action = g_hash_table_lookup(legal, toolbar_buttons[item].pixmap_id);
-                if (action && !g_action_map_lookup_action(map, action))
-                    g_print("undefined action: “%s”\n", action);
-            }
-        }
-#endif /* BALSA_TOOLBAR_DEBUG_ACTIONS */
         if (item > 0
             && (!g_hash_table_lookup(legal,
                                      toolbar_buttons[item].pixmap_id)
@@ -833,24 +785,29 @@ tp_page_swap_rows(ToolbarPage * page, gboolean forward)
 static void
 tp_store_set(GtkListStore * store, GtkTreeIter * iter, gint item)
 {
-    GdkPixbuf *pixbuf;
+    GdkPixbuf *pixbuf = NULL;
     gchar *text;
 
     text = g_strdup(balsa_toolbar_button_text(item));
-    replace_nl_with_space(text);
-    pixbuf =
-        (item > 0
-         ? gtk_icon_theme_load_icon(gtk_icon_theme_get_default(),
-                                    balsa_icon_id(toolbar_buttons[item].pixmap_id),
-                                    GTK_ICON_SIZE_LARGE_TOOLBAR, 0, NULL)
-         : NULL);
+    g_strdelimit(text, "\n", ' ');
+
+    if (item > 0) {
+        const gchar *icon_id;
+
+        icon_id = balsa_icon_id(toolbar_buttons[item].pixmap_id);
+        if (icon_id != NULL)
+            pixbuf =
+                gtk_icon_theme_load_icon(gtk_icon_theme_get_default(), icon_id,
+                                         GTK_ICON_SIZE_LARGE_TOOLBAR, 0, NULL);
+    }
+
     gtk_list_store_set(store, iter,
                        TP_TEXT_COLUMN, text,
                        TP_ICON_COLUMN, pixbuf,
                        TP_ITEM_COLUMN, item,
                        -1);
     g_free(text);
-    if (pixbuf)
+    if (pixbuf != NULL)
         g_object_unref(pixbuf);
 }
 
@@ -889,11 +846,7 @@ tp_page_add_selected(ToolbarPage * page)
 
     gtk_widget_set_sensitive(page->standard_button, TRUE);
     tp_page_refresh_preview(page);
-#ifndef BALSA_TOOLBAR_DEBUG_ACTIONS
     tp_page_refresh_available(page);
-#else /* BALSA_TOOLBAR_DEBUG_ACTIONS */
-    tp_page_refresh_available(page, NULL);
-#endif /* BALSA_TOOLBAR_DEBUG_ACTIONS */
     balsa_toolbar_model_changed(page->model);
 }
 
@@ -924,22 +877,6 @@ tp_page_remove_selected(ToolbarPage * page)
 
     gtk_widget_set_sensitive(page->standard_button, TRUE);
     tp_page_refresh_preview(page);
-#ifndef BALSA_TOOLBAR_DEBUG_ACTIONS
     tp_page_refresh_available(page);
-#else /* BALSA_TOOLBAR_DEBUG_ACTIONS */
-    tp_page_refresh_available(page, NULL);
-#endif /* BALSA_TOOLBAR_DEBUG_ACTIONS */
     balsa_toolbar_model_changed(page->model);
 }
-
-/* Unwrap text.
- */
-static void
-replace_nl_with_space(char* str)
-{
-    while(*str) {
-	if(*str == '\n')
-	    *str=' ';
-	str++;
-    }
-}
